//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include <zlib/zlib.h>
extern "C" {
#include <zlib/zutil.h>
}
#include "updownclient.h"
#include "partfile.h"
#include "opcodes.h"
#include "packets.h"
#include "emule.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//	members of CUpDownClient
//	which are mainly used for downloading functions 

// [TPT]
// Maella -Code Improvement-
void CUpDownClient::DrawStatusBar(CDC* dc, RECT* rect, bool onlygreyrect, bool  bFlat){ 
						   
	const COLORREF crBoth = (bFlat) ? RGB(0, 150, 0) : RGB(0, 192, 0); // Flat => Green
	const COLORREF crNeither = (bFlat) ? RGB(224, 224, 224) : RGB(240, 240, 240); // Flat => Grey
	const COLORREF crClientOnly = (bFlat) ? RGB(0, 0, 0) : RGB(104, 104, 104); // Flat => Black
	const COLORREF crPending = (bFlat) ? RGB(255,208,0) : RGB(255, 208, 0); // Flat => yellow-orange
	const COLORREF crNextPending = (bFlat) ? RGB(255,255,100) : RGB(255,255,100); // Flat => yellow
	const COLORREF crMeOnly = (bFlat) ? RGB(112,112,112) : RGB(172,172,172); //--- [TPT] - xrmb:seeTheNeed ---

	// Set size and fill with default color (grey)
	ASSERT(reqfile);
	CBarShader statusBar(rect->bottom - rect->top, rect->right - rect->left);
	statusBar.SetFileSize((reqfile != NULL) ? (reqfile->GetFileSize()-1) : 1); 
	statusBar.Fill(crNeither); 

	if(onlygreyrect == false && reqfile != NULL && m_abyPartStatus > 0) { 
		// Create a linear array with all pending blocks
		// Remark: The class std::vector has a much more powerfull constructor than the MFC CArray
		std::vector<bool> gettingParts(m_nPartCount, false);
		for(POSITION pos = m_PendingBlocks_list.GetHeadPosition(); pos != NULL; ){
			Requested_Block_Struct* cur_block = m_PendingBlocks_list.GetNext(pos)->block;
			gettingParts[cur_block->StartOffset / PARTSIZE] = true;
		}

		// Fill each block with a color matching the status
		for(uint16 i = 0; i < m_nPartCount; i++){ 
			if(m_abyPartStatus[i] != 0){ 
				const uint32 uStart = PARTSIZE*i;
				const uint32 uEnd = (reqfile->GetFileSize()-1 <= (uStart+PARTSIZE-1)) ? (reqfile->GetFileSize()-1) : (uStart+PARTSIZE-1);

				if(reqfile->IsComplete(uStart, uEnd) == true) 
					statusBar.FillRange(uStart, uEnd, crBoth);
				else if(m_nDownloadState == DS_DOWNLOADING && m_nLastBlockOffset < uEnd &&
				        m_nLastBlockOffset >= uStart)
					statusBar.FillRange(uStart, uEnd, crPending); 
				else if(gettingParts[i] == true)
					statusBar.FillRange(uStart, uEnd, crNextPending); 
				else
					statusBar.FillRange(uStart, uEnd, crClientOnly); 
			} 
			//--- [TPT] - xrmb:seeTheNeed ---
			else if (reqfile->IsComplete(PARTSIZE*i,PARTSIZE*(i+1)-1)){ 
				uint32 uEnd;
				if (PARTSIZE*(i+1) > reqfile->GetFileSize()) 
					uEnd = reqfile->GetFileSize(); 
				else 
					uEnd = PARTSIZE*(i+1); 

				statusBar.FillRange(PARTSIZE*i, uEnd, crMeOnly);
			}
			//--- [TPT] - xrmb:seeTheNeed ---
		} 
	} 

	// Finally draw the graphical object
	statusBar.Draw(dc, rect->left, rect->top, bFlat); 
} 
// Maella end

bool CUpDownClient::Compare(CUpDownClient* tocomp, bool bIgnoreUserhash){
	if(!bIgnoreUserhash && HasValidHash() && tocomp->HasValidHash())
	    return !md4cmp(this->GetUserHash(), tocomp->GetUserHash());
	if (HasLowID()){
        if (GetUserIDHybrid()!=0
			&& GetUserIDHybrid() == tocomp->GetUserIDHybrid()
			&& GetServerIP()!=0
			&& GetServerIP() == tocomp->GetServerIP()
			&& GetServerPort()!=0
			&& GetServerPort() == tocomp->GetServerPort())
            return true;
		if (GetIP()!=0
			&& GetIP() == tocomp->GetIP()
			&& GetUserPort()!=0
			&& GetUserPort() == tocomp->GetUserPort())
          return true;
        if (GetIP()!=0 
			&& GetIP() == tocomp->GetIP() 
			&& GetKadPort()!=0 
			&& GetKadPort() == tocomp->GetKadPort())
            return true;
        return false;
    }
    if (GetUserPort()!=0){
        if (GetUserIDHybrid() == tocomp->GetUserIDHybrid()
			&& GetUserPort() == tocomp->GetUserPort())
            return true;
        if (GetIP()!=0
			&& GetIP() == tocomp->GetIP()
			&& GetUserPort() == tocomp->GetUserPort())
            return true;
    }
	if(GetKadPort()!=0){
		if (GetUserIDHybrid() == tocomp->GetUserIDHybrid()
			&& GetKadPort() == tocomp->GetKadPort())
			return true;
		if( GetIP()!=0
			&& GetIP() == tocomp->GetIP()
			&& GetKadPort() == tocomp->GetKadPort())
			return true;
	}
	return false;
}

void CUpDownClient::AskForDownload(){
	if (theApp.listensocket->TooManySockets() && !(socket && socket->IsConnected()) ){
		if (GetDownloadState() != DS_TOOMANYCONNS)
			SetDownloadState(DS_TOOMANYCONNS);
		return;
	}

	// [TPT]
	// Maella -Unnecessary Protocol Overload-
	// Delay the next refresh of the download session initiated from CPartFile::Process()
	m_dwLastAskedTime = ::GetTickCount();
	m_dwNextTCPAskedTime = m_dwLastAskedTime + GetJitteredFileReaskTime();
	// Maella end
	
	SetDownloadState(DS_CONNECTING);
	TryToConnect();

}

bool CUpDownClient::IsSourceRequestAllowed() {
	DWORD dwTickCount = ::GetTickCount() + CONNECTION_LATENCY;
	unsigned int nTimePassedClient = dwTickCount - GetLastSrcAnswerTime();
	unsigned int nTimePassedFile   = dwTickCount - reqfile->GetLastAnsweredTime();
	bool bNeverAskedBefore = GetLastAskedForSources() == 0;

	UINT uSources = reqfile->GetSourceCount();
	return (
	         //if client has the correct extended protocol
	         ExtProtocolAvailable() && m_bySourceExchangeVer > 1 &&
	         //AND if we need more sources
	         theApp.glob_prefs->GetMaxSourcePerFileSoft() > uSources &&
	         //AND if...
	         (
	           //source is not complete and file is rare, allow once every 10 minutes
	           ( !m_bCompleteSource &&
	             ( uSources - reqfile->GetValidSourcesCount() <= RARE_FILE / 4 ||
			       uSources <= RARE_FILE * 2
			     ) &&
	             (bNeverAskedBefore || nTimePassedClient > SOURCECLIENTREASK)
	           ) ||
	           // otherwise, allow every 90 minutes, but only if we haven't
	           //   asked someone else in last 10 minutes
			   ( (bNeverAskedBefore || nTimePassedClient > (unsigned)(SOURCECLIENTREASK * reqfile->GetCommonFilePenalty())) &&
		         (nTimePassedFile > SOURCECLIENTREASK)
	           )
	         )
	       );
}

void CUpDownClient::SendFileRequest(){
	ASSERT(reqfile != NULL);
	if(!reqfile)
		return;
	AddAskedCountDown();

	CSafeMemFile dataFileReq(16+16);
	dataFileReq.Write(reqfile->GetFileHash(),16);
	if( GetExtendedRequestsVersion() > 0 ){
		reqfile->WritePartStatus(&dataFileReq);
	}
	if( GetExtendedRequestsVersion() > 1 ){
		reqfile->WriteCompleteSourcesCount(&dataFileReq);		// #zegzav:completesrc (add)
	}
	Packet* packet = new Packet(&dataFileReq);
	packet->opcode=OP_FILEREQUEST;
	theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
	socket->SendPacket(packet, true);

	// 26-Jul-2003: removed requesting the file status for files <= PARTSIZE for better compatibility with ed2k protocol (eDonkeyHybrid).
	// if the remote client answers the OP_FILEREQUEST with OP_FILEREQANSWER the file is shared by the remote client. if we
	// know that the file is shared, we know also that the file is complete and don't need to request the file status.
	if (reqfile->GetPartCount() > 1){
	    CSafeMemFile dataSetReqFileID(16);
	    dataSetReqFileID.Write(reqfile->GetFileHash(),16);
	    packet = new Packet(&dataSetReqFileID);
	    packet->opcode = OP_SETREQFILEID;
	    theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
	    socket->SendPacket(packet, true);
	}
	// [TPT]
	// Maella -Unnecessary Protocol Overload-
	// Delay the next refresh of the download session initiated from CPartFile::Process() 
	m_dwLastAskedTime = ::GetTickCount();
	m_dwNextTCPAskedTime = m_dwLastAskedTime + GetJitteredFileReaskTime();
	// Maella end

	if( IsEmuleClient() ){
		SetRemoteQueueFull( true );
		SetRemoteQueueRank(0);
	}	
	if(theApp.glob_prefs->GetDisabledXS() == false && IsSourceRequestAllowed() == true){ // Maella -Enable/Disable source exchange in preference- (Tarod)	   
		reqfile->SetLastAnsweredTimeTimeout();
		Packet* packet = new Packet(OP_REQUESTSOURCES,16,OP_EMULEPROT);
		md4cpy(packet->pBuffer,reqfile->GetFileHash());
		theApp.uploadqueue->AddUpDataOverheadSourceExchange(packet->size);
		socket->SendPacket(packet,true,true);
		SetLastAskedForSources();
		if ( theApp.glob_prefs->GetDebugSourceExchange() )
			AddDebugLogLine( false, "Send:Source Request User(%s) File(%s)", GetUserName(), reqfile->GetFileName() );
	}
}

void CUpDownClient::SendStartupLoadReq(){
	if (socket==NULL || reqfile==NULL){
		ASSERT(0);
		return;
	}
	SetDownloadState(DS_ONQUEUE);
	CSafeMemFile dataStartupLoadReq(16);
	dataStartupLoadReq.Write(reqfile->GetFileHash(),16);
	Packet* packet = new Packet(&dataStartupLoadReq);
	packet->opcode = OP_STARTUPLOADREQ;
	theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
	socket->SendPacket(packet, true, true);
	
	// [TPT]
	// Maella -Unnecessary Protocol Overload-
	// Remark: force a TCP refresh of the download session in 2 hours
	m_dwNextTCPAskedTime = m_dwLastAskedTime + 6 * GetJitteredFileReaskTime();
	// Maella end
	
	// [TPT] Maella -Auto A4AF-
	// Keep a track when this file was asked for the last time to avoid a Ban()
	m_partStatusMap[reqfile].dwStartUploadReqTime = GetTickCount();
	// Maella end
}

void CUpDownClient::ProcessFileInfo(char* packet,uint32 size){
	CSafeMemFile data((BYTE*)packet,size);
	uchar cfilehash[16];
	data.Read(cfilehash,16);
	uint16 namelength;
	data.Read(&namelength,2);
	data.Read(m_strClientFilename.GetBuffer(namelength),namelength);
	m_strClientFilename.ReleaseBuffer(namelength);

	if (reqfile==NULL)
		throw GetResString(IDS_ERR_WRONGFILEID) + _T(" (ProcessFileInfo; reqfile==NULL)");
	if (md4cmp(cfilehash,reqfile->GetFileHash()))
		throw GetResString(IDS_ERR_WRONGFILEID) + _T(" (ProcessFileInfo; reqfile!=cfilehash)");

	// 26-Jul-2003: removed requesting the file status for files <= PARTSIZE for better compatibility with ed2k protocol (eDonkeyHybrid).
	// if the remote client answers the OP_FILEREQUEST with OP_FILEREQANSWER the file is shared by the remote client. if we
	// know that the file is shared, we know also that the file is complete and don't need to request the file status.
	if (reqfile->GetPartCount() == 1){
		if (m_abyPartStatus){
			delete[] m_abyPartStatus;
			m_abyPartStatus = NULL;
		}

		m_nPartCount = reqfile->GetPartCount();
		m_abyPartStatus = new uint8[m_nPartCount];
		MEMSET(m_abyPartStatus,1,m_nPartCount);
		m_bCompleteSource = true;

		UpdateDisplayedInfo();
		reqfile->UpdateAvailablePartsCount();

		// even if the file is <= PARTSIZE, we _may_ need the hashset for that file (if the file size == PARTSIZE)
		if (reqfile->hashsetneeded){
			Packet* packet = new Packet(OP_HASHSETREQUEST,16);
			md4cpy(packet->pBuffer,reqfile->GetFileHash());
			theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
			socket->SendPacket(packet,true,true);
			SetDownloadState(DS_REQHASHSET);
			m_fHashsetRequesting = 1;
			reqfile->hashsetneeded = false;
		}
		else{
			SendStartupLoadReq();
		}
		reqfile->NewSrcPartsInfo();
	}
	// [TPT] - SLUGFILLER: showComments - send comment while you're at it.
	if(m_byAcceptCommentVer >= 2)
		SendCommentInfo(reqfile);
	// [TPT] - SLUGFILLER: showComments
}

void CUpDownClient::ProcessFileStatus(char* packet,uint32 size){
	CSafeMemFile data((BYTE*)packet,size);
	uchar cfilehash[16];
	data.Read(cfilehash,16);
	if ( (!reqfile) || md4cmp(cfilehash,reqfile->GetFileHash())){
		if (reqfile==NULL)
			throw GetResString(IDS_ERR_WRONGFILEID) + _T(" (ProcessFileStatus; reqfile==NULL)");
		throw GetResString(IDS_ERR_WRONGFILEID) + _T(" (ProcessFileStatus; reqfile!=cfilehash)");
	}
	uint16 nED2KPartCount;
	data.Read(&nED2KPartCount,2);
	if (m_abyPartStatus){
		delete[] m_abyPartStatus;
		m_abyPartStatus = NULL;
	}
	bool bPartsNeeded = false;
	if (!nED2KPartCount){
		m_nPartCount = reqfile->GetPartCount();
		m_abyPartStatus = new uint8[m_nPartCount];
		MEMSET(m_abyPartStatus,1,m_nPartCount);
		bPartsNeeded = true;
		m_bCompleteSource = true;
	}
	else{
		if (reqfile->GetED2KPartCount() != nED2KPartCount){
			m_nPartCount = 0;
			throw GetResString(IDS_ERR_WRONGPARTNUMBER);
		}
		m_nPartCount = reqfile->GetPartCount();

		m_bCompleteSource = false;
		m_abyPartStatus = new uint8[m_nPartCount];
		uint16 done = 0;
		while (done != m_nPartCount){
			uint8 toread;
			data.Read(&toread,1);
			for (sint32 i = 0;i != 8;i++){
				m_abyPartStatus[done] = ((toread>>i)&1)? 1:0; 	
				if (m_abyPartStatus[done] && !reqfile->IsComplete(done*PARTSIZE,((done+1)*PARTSIZE)-1))
					bPartsNeeded = true;
				done++;
				if (done == m_nPartCount)
					break;
			}
		}
	}

	//theApp.emuledlg->transferwnd.downloadlistctrl.UpdateItem(this);
	UpdateDisplayedInfo();
	reqfile->UpdateAvailablePartsCount();

	if (!bPartsNeeded)
		SetDownloadState(DS_NONEEDEDPARTS);
	else if (reqfile->hashsetneeded){
		Packet* packet = new Packet(OP_HASHSETREQUEST,16);
		md4cpy(packet->pBuffer,reqfile->GetFileHash());
		theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
		socket->SendPacket(packet, true, true);
		SetDownloadState(DS_REQHASHSET);
		m_fHashsetRequesting = 1;
		reqfile->hashsetneeded = false;
	}
	else{
		SendStartupLoadReq();
	}
	reqfile->NewSrcPartsInfo();
}

// [TPT]
// Maella -Code Improvement-
bool CUpDownClient::AddRequestForAnotherFile(CPartFile* file){
	if(m_OtherRequests_list.Find(file) != NULL) return false; // Found
	if(m_OtherNoNeeded_list.Find(file) != NULL) return false; // Found
	m_OtherRequests_list.AddTail(file);
	file->A4AFsrclist.AddTail(this); // [enkeyDEV(Ottavio84) -A4AF-]

	return true;
}
// Maella end

void CUpDownClient::SetDownloadState(EDownloadState nNewState, DownStopReason reason){ // Maella -Download Stop Reason-
	if (m_nDownloadState != nNewState){
		if(nNewState == DS_DOWNLOADING){
			reqfile->AddDownloadingSource(this);
			theApp.emuledlg->transferwnd.downloadclientsctrl.AddClient(this);//[TPT] - TBH Transfers Window Style
		}
		else if(m_nDownloadState == DS_DOWNLOADING){
			reqfile->RemoveDownloadingSource(this);
		}

		if (m_nDownloadState == DS_DOWNLOADING ){
			theApp.emuledlg->transferwnd.downloadclientsctrl.RemoveClient(this);//[TPT] - TBH Transfers Window Style
			// -khaos--+++> Extended Statistics (Successful/Failed Download Sessions)
			if ( m_bTransferredDownMini && nNewState != DS_ERROR ){
				CUpDownClient::AddDownStopCount(false, reason); // Maella -Download Stop Reason-
				theApp.glob_prefs->Add2DownSuccessfulSessions(); // Increment our counters for successful sessions (Cumulative AND Session)
			}
			else {
				CUpDownClient::AddDownStopCount(true, reason); // Maella -Download Stop Reason-
				theApp.glob_prefs->Add2DownFailedSessions(); // Increment our counters failed sessions (Cumulative AND Session)
			}
			theApp.glob_prefs->Add2DownSAvgTime(GetDownTimeDifference()/1000);
			// <-----khaos-

			m_nDownloadState = nNewState;
			for (POSITION pos = m_DownloadBlocks_list.GetHeadPosition();pos != 0;m_DownloadBlocks_list.GetNext(pos)){
				Requested_Block_Struct* cur_block = m_DownloadBlocks_list.GetAt(pos);
				reqfile->RemoveBlockFromList(cur_block->StartOffset,cur_block->EndOffset);
				delete m_DownloadBlocks_list.GetAt(pos);
			}
			m_DownloadBlocks_list.RemoveAll();

			for (POSITION pos = m_PendingBlocks_list.GetHeadPosition();pos != 0;m_PendingBlocks_list.GetNext(pos))
			{
				Pending_Block_Struct *pending = m_PendingBlocks_list.GetAt(pos);
				if (reqfile)
					reqfile->RemoveBlockFromList(pending->block->StartOffset, pending->block->EndOffset);
				
				delete pending->block;
				// Not always allocated
				if (pending->zStream){
					inflateEnd(pending->zStream);
					delete pending->zStream;
				}
				delete pending;
			}
			m_PendingBlocks_list.RemoveAll();

			// Maella -Accurate measure of bandwidth: eDonkey data + control, network adapter-
			m_nDownDatarate = 0;
			m_nDownDatarateMeasure = 0;
			m_sumDownHistory = 0;
			m_downHistory_list.RemoveAll();
			// Maella end

			// Maella -Code Improvement-
			if(nNewState == DS_ONQUEUE){
				// The downloading session was terminated by a OP_OUTOFPARTREQS (e.g. Hybrid, shareaza)
				// or a Disconnect() (e.g. eMule). In this second case it might be necessary to reask if
				// the disconnection was caused by a timeout, otherwise we could be thrown out of the 
				// remote waiting list.				
				if(reqfile != NULL){
					// Reask for the file as soon as possible.
					TrigNextSafeAskForDownload(reqfile);
				}
			}
			// Maella end

			if (nNewState == DS_NONE){
				if (m_abyPartStatus)
					delete[] m_abyPartStatus;
				m_abyPartStatus = NULL;
				m_nPartCount = 0;
			}
			if (socket && nNewState != DS_ERROR)
				socket->DisableDownloadLimit();
		}
		m_nDownloadState = nNewState;
		if( GetDownloadState() == DS_DOWNLOADING ){
			if ( IsEmuleClient() )
				SetRemoteQueueFull(false);
			SetRemoteQueueRank(0);
			SetAskedCountDown(0);
		}
		UpdateDisplayedInfo(true);
	}
}

void CUpDownClient::ProcessHashSet(char* packet,uint32 size){
	if (!m_fHashsetRequesting)
		throw CString(_T("unwanted hashset"));
	if ( (!reqfile) || md4cmp(packet,reqfile->GetFileHash()))
		throw GetResString(IDS_ERR_WRONGFILEID) + _T(" (ProcessHashSet)");	
	CSafeMemFile data((BYTE*)packet,size);
	if (reqfile->LoadHashsetFromFile(&data,true)){
		m_fHashsetRequesting = 0;
	}
	else{
		reqfile->hashsetneeded = true;
		throw GetResString(IDS_ERR_BADHASHSET);
	}
	SendStartupLoadReq();
}

void CUpDownClient::SendBlockRequests(){
	m_dwLastBlockReceived = ::GetTickCount();
	if (!reqfile)
		return;
	if (m_DownloadBlocks_list.IsEmpty())
	{
		// Barry - instead of getting 3, just get how many is needed
		uint16 count = 3 - m_PendingBlocks_list.GetCount();
		Requested_Block_Struct** toadd = new Requested_Block_Struct*[count];
		if (reqfile->GetNextRequestedBlock(this,toadd,&count)){
			for (int i = 0; i < count; i++)
				m_DownloadBlocks_list.AddTail(toadd[i]);			
		}
		delete[] toadd;
	}

	// Barry - Why are unfinished blocks requested again, not just new ones?

	while (m_PendingBlocks_list.GetCount() < 3 && !m_DownloadBlocks_list.IsEmpty()){
		Pending_Block_Struct* pblock = new Pending_Block_Struct;
		pblock->block = m_DownloadBlocks_list.RemoveHead();
		pblock->zStream = NULL;
		pblock->totalUnzipped = 0;
		pblock->bZStreamError = false;
		m_PendingBlocks_list.AddTail(pblock);
	}
	if (m_PendingBlocks_list.IsEmpty()){
		Packet* packet = new Packet(OP_CANCELTRANSFER,0);
		theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
		socket->SendPacket(packet,true,true);
		// [TPT]
		SetDownloadState(DS_NONEEDEDPARTS, CUpDownClient::DSR_NONEEDEDPARTS); // Maella -Download Stop Reason-
		return;
	}
	const int iPacketSize = 16+(3*4)+(3*4); // 40
	Packet* packet = new Packet(OP_REQUESTPARTS,iPacketSize);
	CSafeMemFile data((BYTE*)packet->pBuffer,iPacketSize);
	data.Write(reqfile->GetFileHash(),16);
	POSITION pos = m_PendingBlocks_list.GetHeadPosition();
	uint32 null = 0;
	Requested_Block_Struct* block;
	for (uint32 i = 0; i != 3; i++){
		if (pos){
			block = m_PendingBlocks_list.GetAt(pos)->block;
			m_PendingBlocks_list.GetNext(pos);
			data.Write(&block->StartOffset,4);
		}
		else
			data.Write(&null,4);
	}
	pos = m_PendingBlocks_list.GetHeadPosition();
	for (uint32 i = 0; i != 3; i++){
		if (pos){
			block = m_PendingBlocks_list.GetAt(pos)->block;
			m_PendingBlocks_list.GetNext(pos);
			uint32 endpos = block->EndOffset+1;
			data.Write(&endpos,4);
		}
		else
			data.Write(&null,4);
	}
	theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
	socket->SendPacket(packet,true,true);
}

/* Barry - Originally this only wrote to disk when a full 180k block 
           had been received from a client, and only asked for data in 
		   180k blocks.

		   This meant that on average 90k was lost for every connection
		   to a client data source. That is a lot of wasted data.

		   To reduce the lost data, packets are now written to a buffer
		   and flushed to disk regularly regardless of size downloaded.
		   This includes compressed packets.

		   Data is also requested only where gaps are, not in 180k blocks.
		   The requests will still not exceed 180k, but may be smaller to
		   fill a gap.
*/
void CUpDownClient::ProcessBlockPacket(char *packet, uint32 size, bool packed)
{
  try {
	// Ignore if no data required
	if (!(GetDownloadState() == DS_DOWNLOADING || GetDownloadState() == DS_NONEEDEDPARTS))
		return;

	theApp.emuledlg->transferwnd.downloadclientsctrl.RefreshClient(this);//[TPT] - TBH Transfers Window Style
	const int HEADER_SIZE = 24;

	// Update stats
	m_dwLastBlockReceived = ::GetTickCount();


	// Read data from packet
	CSafeMemFile data((BYTE*)packet, size);
	uchar fileID[16];
	data.Read(fileID, 16);

	// Check that this data is for the correct file
	if ( (!reqfile) || md4cmp(packet, reqfile->GetFileHash()))
	{
		throw GetResString(IDS_ERR_WRONGFILEID) + _T(" (ProcessBlockPacket)");
	}

	// Find the start & end positions, and size of this chunk of data
	uint32 nStartPos;
	uint32 nEndPos;
	uint32 nBlockSize = 0;
	data.Read(&nStartPos, 4);
	if (packed)
	{
		data.Read(&nBlockSize, 4);
		nEndPos = nStartPos + (size - HEADER_SIZE);
		usedcompressiondown = true;
	}
	else
		data.Read(&nEndPos,4);

	// Check that packet size matches the declared data size + header size (24)
	if ( size != ((nEndPos - nStartPos) + HEADER_SIZE))
		throw GetResString(IDS_ERR_BADDATABLOCK) + _T(" (ProcessBlockPacket)");

	// -khaos--+++>
	// Extended statistics information based on which client and remote port sent this data.
	// The new function adds the bytes to the grand total as well as the given client/port.
	// bFromPF is not relevant to downloaded data.  It is purely an uploads statistic.

	theApp.glob_prefs->Add2SessionTransferData(GetClientSoft(), GetUserPort(), false, false, size - HEADER_SIZE);
	// <-----khaos-

	//m_nDownDataRateMS += size - HEADER_SIZE; // [TPT]
	credits->AddDownloaded(size - HEADER_SIZE, GetIP());
	//<<< eWombat [SNAFU][TPT]
	if (theApp.glob_prefs->GetAntiSnafu())
		{
		AddTrustedTransfer(size-HEADER_SIZE); 
		if (GetTrustedTransfer()>3072000)
			{
			SetTrustedTransfer(0);
			if (!IsNotSUI())
				{
				Trust();
				if (IsSnafu())
					UnSnafu(true);
				}
			}
		}
	// >>> eWombat [SNAFU][TPT]

	// Move end back one, should be inclusive
	nEndPos--;

	// Loop through to find the reserved block that this is within
	Pending_Block_Struct *cur_block;
	for (POSITION pos = m_PendingBlocks_list.GetHeadPosition(); pos != NULL; m_PendingBlocks_list.GetNext(pos))
	{
		cur_block = m_PendingBlocks_list.GetAt(pos);
		if ((cur_block->block->StartOffset <= nStartPos) && (cur_block->block->EndOffset >= nStartPos))
		{
			// Found reserved block			
			if (cur_block->bZStreamError){
				AddDebugLogLine(false, _T("Ignoring %u bytes of block %u-%u because of errornous zstream state for file \"%s\""), size - HEADER_SIZE, nStartPos, nEndPos, reqfile->GetFileName());
				reqfile->RemoveBlockFromList(cur_block->block->StartOffset, cur_block->block->EndOffset);
				return;
			}
            
			// Remember this start pos, used to draw part downloading in list
			m_nLastBlockOffset = nStartPos;  

			// Occasionally packets are duplicated, no point writing it twice
			// This will be 0 in these cases, or the length written otherwise
			uint32 lenWritten = 0;

			// Handle differently depending on whether packed or not
			if (!packed)
			{
				// Write to disk (will be buffered in part file class)
				lenWritten = reqfile->WriteToBuffer(size - HEADER_SIZE, 
													(BYTE *) (packet + HEADER_SIZE),
													nStartPos,
													nEndPos,
													cur_block->block );
			}
			else // Packed
			{
				// Create space to store unzipped data, the size is only an initial guess, will be resized in unzip() if not big enough
				uint32 lenUnzipped = (size * 3);  // [TPT]
				// Don't get too big
				if (lenUnzipped > (EMBLOCKSIZE + 300))
					lenUnzipped = (EMBLOCKSIZE + 300);
				BYTE *unzipped = new BYTE[lenUnzipped];

				// Try to unzip the packet
				int result = unzip(cur_block, (BYTE *)(packet + HEADER_SIZE), (size - HEADER_SIZE), &unzipped, &lenUnzipped);
				if (result == Z_OK)
				{
					// Maella -Defeat 0-filled Part Senders- (Idea of xrmb)
					if(theApp.glob_prefs->GetCorruptionTest() == true && 
					   (reqfile->IsCDImage() == false || (reqfile->IsCDImage() == true && theApp.glob_prefs->GetSkipCorruptionCDTest() == false))){
						
						// Check the compression factor
						if(lenUnzipped > 25*nBlockSize || (lenUnzipped > 4*nBlockSize && reqfile->IsArchive() == true)){

							// Format User hash
							CString userHash;
							for(int  i=0; i<16; i++){
								TCHAR buffer[33];
								_stprintf(buffer, _T("%02X"), GetUserHash()[i]);
								userHash += buffer;
							}

							// Log
							AddLogLine(true,  _T("Received suspicious block: file '%s', part %i, block %i, blocksize %i, comp. blocksize %i, comp. factor %0.1f)"), 
														reqfile->GetFileName(), cur_block->block->StartOffset/PARTSIZE, cur_block->block->StartOffset/EMBLOCKSIZE, lenUnzipped, nBlockSize, (float)lenUnzipped/nBlockSize);
							AddLogLine(false, _T("Username '%s' (IP %s:%i), hash %s"), m_pszUsername, GetFullIP(), GetUserPort(), userHash);

							// Ban => serious error (Attack?)
							if(lenUnzipped > 4*nBlockSize && reqfile->IsArchive() == true){
								theApp.ipfilter->AddBannedIPRange(GetIP(), GetIP(), 1, _T("Temporary"));
								theApp.emuledlg->AddPhoenixLogLine(_T("serious error (Attack?) User '%s' (IP %s:%i)  has been added temporary to the filered list"), m_pszUsername, GetFullIP(), GetUserPort()); // eWombat [SNAFU][TPT]
								SetDownloadState(DS_ERROR, CUpDownClient::DSR_CORRUPTEDBLOCK); // Maella -Download Stop Reason-
								
							}

							// Do Not Save
							lenWritten = 0;
							lenUnzipped = 0; // skip writting
							reqfile->RemoveBlockFromList(cur_block->block->StartOffset, cur_block->block->EndOffset);
							AddLogLine(false, _T("Block dropped"));
						}
					}
					// Maella end

					// Write any unzipped data to disk
					if (lenUnzipped > 0)
					{
						// Use the current start and end positions for the uncompressed data
						nStartPos = cur_block->block->StartOffset + cur_block->totalUnzipped - lenUnzipped;
						nEndPos = cur_block->block->StartOffset + cur_block->totalUnzipped - 1;

						// Maella -Overwrite Already Downloaded Parts And Chunk- (emarc)
					        if (nStartPos > cur_block->block->EndOffset || nEndPos > cur_block->block->EndOffset){
							// Format User hash
							CString userHash;
							for(int  i=0; i<16; i++){
								TCHAR buffer[33];
								_stprintf(buffer, _T("%02X"), GetUserHash()[i]);
								userHash += buffer;
							}

							// Log
							AddLogLine(true,  _T("Received block out of limit: file '%s', part %i, block %i, blocksize %i, comp. blocksize %i, comp. factor %0.1f)"), 
														reqfile->GetFileName(), cur_block->block->StartOffset/PARTSIZE, cur_block->block->StartOffset/EMBLOCKSIZE, lenUnzipped, nBlockSize, (float)lenUnzipped/nBlockSize);
							AddLogLine(false, _T("User '%s' (IP %s:%i), hash %s, client=%s has been added temporary to the filered list"), m_pszUsername, GetFullIP(), GetUserPort(), userHash, GetClientSoftFullName());							

							// Ban => serious error (Attack?)
							theApp.ipfilter->AddBannedIPRange(GetIP(), GetIP(), 1, _T("Temporary"));
							SetDownloadState(DS_ERROR, CUpDownClient::DSR_CORRUPTEDBLOCK); // Maella -Download Stop Reason-

							// Do Not Save
							lenWritten = 0;
							lenUnzipped = 0; // skip writting
							reqfile->RemoveBlockFromList(cur_block->block->StartOffset, cur_block->block->EndOffset);
							AddLogLine(false, _T("Block dropped"));
						}
						else{
							// Write uncompressed data to file
							lenWritten = reqfile->WriteToBuffer(size - HEADER_SIZE,
								unzipped,
								nStartPos,
								nEndPos,
								cur_block->block );
						}
						// Maella end
					}
				}
				else
				{
					    CString strZipError;
					    if (cur_block->zStream && cur_block->zStream->msg)
						    strZipError.Format(_T(" - %s"), cur_block->zStream->msg);
					    else
						    strZipError.Format(_T(" - %s"), ERR_MSG(result));
					    AddDebugLogLine(false, GetResString(IDS_ERR_CORRUPTCOMPRPKG) + strZipError, reqfile->GetFileName(), result);
					    reqfile->RemoveBlockFromList(cur_block->block->StartOffset, cur_block->block->EndOffset);
    
					    // If we had an zstream error, there is no chance that we could recover from it nor that we
					    // could use the current zstream (which is in error state) any longer.
					    if (cur_block->zStream){
						    inflateEnd(cur_block->zStream);
						    delete cur_block->zStream;
						    cur_block->zStream = NULL;
					    }
    
					    // Although we can't further use the current zstream, there is no need to disconnect the sending 
					    // client because the next zstream (a series of 10K-blocks which build a 180K-block) could be
					    // valid again. Just ignore all further blocks for the current zstream.
					    cur_block->bZStreamError = true;

					// Maella -Defeat 0-filled Part Senders- (Idea of xrmb)
					CString userHash;
					for(int  i=0; i<16; i++){
						TCHAR buffer[33];
						_stprintf(buffer, _T("%02X"), GetUserHash()[i]);
						userHash += buffer;
					}
					// Ban => serious error (Attack?)
					AddLogLine(false, _T("User '%s' (IP %s:%i), hash %s, client=%s has been added temporary to the filered list"), m_pszUsername, GetFullIP(), GetUserPort(), userHash, GetClientSoftFullName());
					theApp.ipfilter->AddBannedIPRange(GetIP(), GetIP(), 1, _T("Temporary"));
					SetDownloadState(DS_ERROR, CUpDownClient::DSR_CORRUPTEDBLOCK); // Maella -Download Stop Reason-
					// Maella end
				}
				delete [] unzipped;
			}

			// These checks only need to be done if any data was written
			if (lenWritten > 0)
			{
				m_nTransferedDown += lenWritten;
				
				SetTransferredDownMini(); // Sets boolean m_bTransferredDownMini to true // -khaos--+++> For determining whether the current download session was a success or not.

				// If finished reserved block
				if (nEndPos == cur_block->block->EndOffset)
				{
					reqfile->RemoveBlockFromList(cur_block->block->StartOffset, cur_block->block->EndOffset);
					delete cur_block->block;
					// Not always allocated
					if (cur_block->zStream){
						inflateEnd(cur_block->zStream);
						delete cur_block->zStream;
					}
					delete cur_block;
					m_PendingBlocks_list.RemoveAt(pos);

					// Request next block
					SendBlockRequests();	
				}
			}

			// Stop looping and exit method
			return;
		}
	}
  } catch (...) {
	  AddDebugLogLine(false, _T("Unknown exception in %s: file \"%s\""), __FUNCTION__, reqfile ? reqfile->GetFileName() : NULL);
	}

}

int CUpDownClient::unzip(Pending_Block_Struct *block, BYTE *zipped, uint32 lenZipped, BYTE **unzipped, uint32 *lenUnzipped, int iRecursion)
{
#define TRACE_UNZIP	/*TRACE*/

	TRACE_UNZIP("unzip: Zipd=%6u Unzd=%6u Rcrs=%d", lenZipped, *lenUnzipped, iRecursion);

  	int err = Z_DATA_ERROR;
  	try
  	{
	    // Save some typing
	    z_stream *zS = block->zStream;
    
	    // Is this the first time this block has been unzipped
	    if (zS == NULL)
	    {
		    // Create stream
		    block->zStream = new z_stream;
		    zS = block->zStream;
    
		    // Initialise stream values
		    zS->zalloc = (alloc_func)0;
		    zS->zfree = (free_func)0;
		    zS->opaque = (voidpf)0;
    
		    // Set output data streams, do this here to avoid overwriting on recursive calls
		    zS->next_out = (*unzipped);
		    zS->avail_out = (*lenUnzipped);
    
		    // Initialise the z_stream
		    err = inflateInit(zS);
			if (err != Z_OK){
				TRACE_UNZIP("; Error: new stream failed: %d\n", err);
			    return err;
	    }
		}
	    // Use whatever input is provided
	    zS->next_in  = zipped;
	    zS->avail_in = lenZipped;
    
	    // Only set the output if not being called recursively
	    if (iRecursion == 0)
	    {
		    zS->next_out = (*unzipped);
		    zS->avail_out = (*lenUnzipped);
	    }
    
	    // Try to unzip the data
		TRACE_UNZIP("; inflate(ain=%6u tin=%6u aout=%6u tout=%6u)", zS->avail_in, zS->total_in, zS->avail_out, zS->total_out);
	    err = inflate(zS, Z_SYNC_FLUSH);
    
	    // Is zip finished reading all currently available input and writing all generated output
	    if (err == Z_STREAM_END)
	    {
		    // Finish up
		    err = inflateEnd(zS);
			if (err != Z_OK){
				TRACE_UNZIP("; Error: end stream failed: %d\n", err);
			    return err;
			}
			TRACE_UNZIP("; Z_STREAM_END\n");

    
		    // Got a good result, set the size to the amount unzipped in this call (including all recursive calls)
		    (*lenUnzipped) = (zS->total_out - block->totalUnzipped);
		    block->totalUnzipped = zS->total_out;
	    }
	    else if ((err == Z_OK) && (zS->avail_out == 0) && (zS->avail_in != 0))
	    {
			// [TPT] - Xman (Pawcio)
			// unzipped data is greater than *lenUnzipped, something has gone wrong, return
			if (zS->total_out - block->totalUnzipped > *lenUnzipped) {//Pawcio: UnZip Fix
				theApp.emuledlg->AddDebugLogLine(false,"Unexpected zip error in file \"%s\" (data greater than buffer)", reqfile ? reqfile->GetFileName() : NULL);
				return Z_BUF_ERROR;
			}

		    // Output array was not big enough, call recursively until there is enough space
    		TRACE_UNZIP("; output array not big enough (ain=%u)\n", zS->avail_in);

		    // What size should we try next
		    uint32 newLength = (*lenUnzipped) *= 2;
		    if (newLength == 0)
			    newLength = lenZipped * 2;
    
		    // Copy any data that was successfully unzipped to new array
		    BYTE *temp = new BYTE[newLength];
			ASSERT( zS->total_out - block->totalUnzipped <= newLength );
		    MEMCOPY(temp, (*unzipped), (zS->total_out - block->totalUnzipped));
		    delete [] (*unzipped);
		    (*unzipped) = temp;
		    (*lenUnzipped) = newLength;
    
		    // Position stream output to correct place in new array
		    zS->next_out = (*unzipped) + (zS->total_out - block->totalUnzipped);
		    zS->avail_out = (*lenUnzipped) - (zS->total_out - block->totalUnzipped);
    
		    // Try again
		    err = unzip(block, zS->next_in, zS->avail_in, unzipped, lenUnzipped, iRecursion + 1);
	    }
	    else if ((err == Z_OK) && (zS->avail_in == 0))
	    {
			TRACE_UNZIP("; all input processed\n");
		    // All available input has been processed, everything ok.
		    // Set the size to the amount unzipped in this call (including all recursive calls)
		    (*lenUnzipped) = (zS->total_out - block->totalUnzipped);
		    block->totalUnzipped = zS->total_out;
	    }
	    else
	    {
		    // Should not get here unless input data is corrupt
			

			CString strZipError;
			if (zS->msg)
				strZipError.Format(_T(" '%s'"), zS->msg);
			else if (err != Z_OK)
				strZipError.Format(_T(" %d: '%s'"), err, ERR_MSG(err));
			TRACE_UNZIP("; Error: %s\n", strZipError);
			AddDebugLogLine(false,"Unexpected zip error%s in file \"%s\"", strZipError, reqfile ? reqfile->GetFileName() : NULL);
	    }
    
	    if (err != Z_OK)
		    (*lenUnzipped) = 0;
  	}
  	catch (...){
		AddDebugLogLine(false, _T("Unknown exception in %s: file \"%s\""), __FUNCTION__, reqfile ? reqfile->GetFileName() : NULL);
		err = Z_DATA_ERROR;
	}
  	return err;
}

// [TPT]
// Maella -Accurate measure of bandwidth: eDonkey data + control, network adapter-
void CUpDownClient::CompDownloadRate(){
	// Add new sample
	TransferredData newItem;
	newItem.dataLength = m_nDownDatarateMeasure;
	newItem.timeStamp  = ::GetTickCount();
	m_downHistory_list.AddTail(newItem);
	m_nDownDatarateMeasure = 0;

	// Remove old sample(s)
	uint32 oldTime = m_downHistory_list.GetHead().timeStamp - 1000; // must not be zero
	while(m_downHistory_list.GetSize() > theApp.glob_prefs->GetDatarateSamples()){
		const TransferredData& oldItem = m_downHistory_list.GetHead();
		m_sumDownHistory -= oldItem.dataLength;	
		oldTime = oldItem.timeStamp;
		m_downHistory_list.RemoveHead();
	}

	// Compute datarate
	m_sumDownHistory += newItem.dataLength;
	uint32 deltaTime = newItem.timeStamp - oldTime; // [ms]
	m_nDownDatarate = (deltaTime > 0) ? (1000 * m_sumDownHistory / deltaTime) : 0;   // [bytes/s]

	// Check and then refresh GUI
	m_displayDownDatarateCounter++;
	if(m_displayDownDatarateCounter >= DISPLAY_REFRESH && GetDownloadState() == DS_DOWNLOADING){
		m_displayDownDatarateCounter = 0;
		theApp.emuledlg->transferwnd.downloadlistctrl.UpdateItem(this);
		theApp.emuledlg->transferwnd.clientlistctrl.RefreshClient(this);
	}
}

void CUpDownClient::CheckDownloadTimeout(){
	if ((::GetTickCount() - m_dwLastBlockReceived) > DOWNLOADTIMEOUT){
		Packet* packet = new Packet(OP_CANCELTRANSFER,0);
		theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
		socket->SendPacket(packet,true,true);
		SetDownloadState(DS_ONQUEUE, CUpDownClient::DSR_TIMEOUT); // Maella -Download Stop Reason-
	}
}
// Maella End

uint16 CUpDownClient::GetAvailablePartCount(){
	uint16 result = 0;
	for (int i = 0;i < m_nPartCount;i++){
		if (IsPartAvailable(i))
			result++;
	}
	return result;
}

void CUpDownClient::SetRemoteQueueRank(uint16 nr){	
	// [TPT] - itsonlyme: displayOptions
	if (nr != m_nRemoteQueueRank)
		m_nDifferenceQueueRank = (nr-m_nRemoteQueueRank);
	// [TPT] - itsonlyme: displayOptions
	m_nRemoteQueueRank = nr;
	UpdateDisplayedInfo();
}

void CUpDownClient::UDPReaskACK(uint16 nNewQR){
	//m_bUDPPending = false; // [TPT]
	SetRemoteQueueRank(nNewQR);
	m_dwLastAskedTime = ::GetTickCount();
}

void CUpDownClient::UDPReaskFNF(){
	//m_bUDPPending = false; // [TPT]
	if (GetDownloadState()!=DS_DOWNLOADING){ // avoid premature deletion of 'this' client
		AddDebugLogLine(false,CString("UDP ANSWER FNF : %s"),GetUserName());
		theApp.downloadqueue->RemoveSource(this);
		if (!socket)
			Disconnected();
	}
	else
		AddDebugLogLine(false,CString("UDP ANSWER FNF : %s - did not remove client because of current download state"),GetUserName());
}

// [TPT]
// Maella -Unnecessary Protocol Overload-
void CUpDownClient::UDPReaskForDownload(){
	// This method can be called only every 30 seconds
	if((reqfile == NULL) || // No file to ping
	   (m_abyPartStatus == NULL) || // File status unknown yet
	   (GetTickCount() - m_dwLastUDPReaskTime < 30000)) // Every 30 seconds only
		return;
	// Time stamp
	m_dwLastUDPReaskTime = GetTickCount();

	//the line "m_bUDPPending = true;" use to be here
	// deadlake PROXYSUPPORT
	const ProxySettings& proxy = theApp.glob_prefs->GetProxy();
	if(m_nUDPPort != 0 && theApp.glob_prefs->GetUDPPort() != 0 &&
	   !theApp.IsFirewalled() && !HasLowID() && !(socket && socket->IsConnected())&& (!proxy.UseProxy)) { // deadlake PROXYSUPPORT

		//don't use udp to ask for sources
		if(IsSourceRequestAllowed())
			return;
		//m_bUDPPending = true; // [TPT]
		uint16 packetsize= 16;
		if (GetUDPVersion() >= 3)
			packetsize+= 2;
		Packet* response = new Packet(OP_REASKFILEPING,packetsize,OP_EMULEPROT);	// #zegzav:completesrc (modify)
		md4cpy(response->pBuffer,reqfile->GetFileHash());
		if (GetUDPVersion() >= 3)
		{
			uint16 completecount= reqfile->m_nCompleteSourcesCount;
			MEMCOPY(response->pBuffer+16, &completecount, 2);
		}
		theApp.uploadqueue->AddUpDataOverheadFileRequest(response->size);
		theApp.clientudp->SendPacket(response,GetIP(),GetUDPPort());
	}
}
// Maella end

// Barry - Sets string to show parts downloading, eg NNNYNNNNYYNYN
void CUpDownClient::ShowDownloadingParts(CString *partsYN)
{
	Requested_Block_Struct *cur_block;
	int x;

	// Initialise to all N's
	char *n = new char[m_nPartCount+1];
	_strnset(n, 'N', m_nPartCount);
	n[m_nPartCount] = 0;
	partsYN->SetString(n, m_nPartCount);
	delete [] n;

	for (POSITION pos = m_PendingBlocks_list.GetHeadPosition(); pos != 0; m_PendingBlocks_list.GetNext(pos))
	{
		cur_block = m_PendingBlocks_list.GetAt(pos)->block;
		x = (cur_block->StartOffset / PARTSIZE);
		partsYN->SetAt(x, 'Y');
	}
}

void CUpDownClient::UpdateDisplayedInfo(boolean force) {
    DWORD curTick = ::GetTickCount();

    if(force || curTick-m_lastRefreshedDLDisplay > MINWAIT_BEFORE_DLDISPLAY_WINDOWUPDATE+(uint32)(rand()/(RAND_MAX/1000))) {
	    theApp.emuledlg->transferwnd.downloadlistctrl.UpdateItem(this);
		theApp.emuledlg->transferwnd.clientlistctrl.RefreshClient(this);
        m_lastRefreshedDLDisplay = curTick;
    }
}


// IgnoreNoNeeded = will switch to files of which this source has no needed parts (if no better fiels found)
// ignoreSuspensions = ignore timelimit for A4Af jumping
// bRemoveCompletely = do not readd the file which the source is swapped from to the A4AF lists (needed if deleting or stopping a file)
// toFile = Try to swap to this partfile only
bool CUpDownClient::SwapToAnotherFile(bool bIgnoreNoNeeded, bool ignoreSuspensions, bool bRemoveCompletely, CPartFile* toFile){
	if (GetDownloadState() == DS_DOWNLOADING)
		return false;

	CPartFile* SwapTo = NULL;
	CPartFile* cur_file = NULL;
	int cur_prio= -1;
	POSITION finalpos = NULL;
	CTypedPtrList<CPtrList, CPartFile*>* usedList;

	if (!m_OtherRequests_list.IsEmpty()){
		usedList = &m_OtherRequests_list;
		for (POSITION pos = m_OtherRequests_list.GetHeadPosition();pos != 0;m_OtherRequests_list.GetNext(pos)){
			cur_file = m_OtherRequests_list.GetAt(pos);
			if (cur_file != reqfile && theApp.downloadqueue->IsPartFile(cur_file) && !cur_file->IsStopped() 
				&& (cur_file->GetStatus(false) == PS_READY || cur_file->GetStatus(false) == PS_EMPTY))	
			{
				if (toFile != NULL){
					if (cur_file == toFile){
						SwapTo = cur_file;
						finalpos = pos;
						break;
					}
				}
				else if ( cur_file->GetDownPriority()>cur_prio 
					&& (ignoreSuspensions  || (!ignoreSuspensions && !IsSwapSuspended(cur_file)) ) )
				{
					SwapTo = cur_file;
					cur_prio=cur_file->GetDownPriority();
					finalpos=pos;
					if (cur_prio==PR_HIGH)
						break;
				}
			}
		}
	}


	if (!SwapTo && bIgnoreNoNeeded){
		usedList = &m_OtherNoNeeded_list;
		for (POSITION pos = m_OtherNoNeeded_list.GetHeadPosition();pos != 0;m_OtherNoNeeded_list.GetNext(pos)){
			cur_file = m_OtherNoNeeded_list.GetAt(pos);
			if (cur_file != reqfile && theApp.downloadqueue->IsPartFile(cur_file) && !cur_file->IsStopped() 
				&& (cur_file->GetStatus(false) == PS_READY || cur_file->GetStatus(false) == PS_EMPTY) )	
			{
				if (toFile != NULL){
					if (cur_file == toFile){
						SwapTo = cur_file;
						finalpos = pos;
						break;
					}
				}
				else if ( cur_file->GetDownPriority()>cur_prio 
					&& (ignoreSuspensions  || (!ignoreSuspensions && !IsSwapSuspended(cur_file)) ) )
				{
					SwapTo = cur_file;
					cur_prio=cur_file->GetDownPriority();
					finalpos=pos;
					if (cur_prio==PR_HIGH)
						break;
				}
			}
		}
	}

	if (SwapTo){
		//AddDebugLogLine(false, "Swapped source '%s'; Status %i; Remove %s to %s", this->GetUserName(), this->GetDownloadState(), (bRemoveCompletely ? "Yes" : "No" ), SwapTo->GetFileName());		
		if (DoSwap(SwapTo,bRemoveCompletely)){

			usedList->RemoveAt(finalpos);
			return true;
		}
	}

	return false;
}

// [TPT]
bool CUpDownClient::DoSwap(CPartFile* SwapTo, bool bRemoveCompletely) {
	POSITION pos = reqfile->srcList.Find(this);
	if(pos)
	{
		// remove this client from the A4AF list of our new reqfile
		POSITION pos2 = SwapTo->A4AFsrclist.Find(this);
		if (pos2){
			SwapTo->A4AFsrclist.RemoveAt(pos2);
			theApp.emuledlg->transferwnd.downloadlistctrl.RemoveSource(this,SwapTo);
		}

		reqfile->srcList.RemoveAt(pos);
		reqfile->RemoveDownloadingSource(this);

		if(!bRemoveCompletely)
		{
			reqfile->A4AFsrclist.AddTail(this);
			if (GetDownloadState() == DS_NONEEDEDPARTS)
				m_OtherNoNeeded_list.AddTail(reqfile);
			else
				m_OtherRequests_list.AddTail(reqfile);

			if (!bRemoveCompletely)
				theApp.emuledlg->transferwnd.downloadlistctrl.AddSource(reqfile,this,true);
		}


		SetDownloadState(DS_NONE);
		ResetFileStatusInfo();
		m_nRemoteQueueRank = 0;
		m_nDifferenceQueueRank = 0;	// [TPT] - itsonlyme: displayOptions
		
		reqfile->NewSrcPartsInfo();
		reqfile->UpdateAvailablePartsCount();
		reqfile = SwapTo;

		SwapTo->srcList.AddTail(this);
		theApp.emuledlg->transferwnd.downloadlistctrl.AddSource(SwapTo,this,false);


		return true;
	}
	return false;
}


void CUpDownClient::DontSwapTo(CPartFile* file) {
	DWORD dwNow = ::GetTickCount();

	for (POSITION pos = m_DontSwap_list.GetHeadPosition(); pos != 0; m_DontSwap_list.GetNext(pos))
		if(m_DontSwap_list.GetAt(pos).file == file) {
			m_DontSwap_list.GetAt(pos).timestamp = dwNow ;
			return;
		}
	PartFileStamp newfs = {file, dwNow };
	m_DontSwap_list.AddHead(newfs);
}

bool CUpDownClient::IsSwapSuspended(CPartFile* file){
	if (m_DontSwap_list.GetCount()==0)
		return false;

	for (POSITION pos = m_DontSwap_list.GetHeadPosition(); pos != 0 && m_DontSwap_list.GetCount()>0; m_DontSwap_list.GetNext(pos)){
		if(m_DontSwap_list.GetAt(pos).file == file){
			if ( ::GetTickCount() - m_DontSwap_list.GetAt(pos).timestamp  >= PURGESOURCESWAPSTOP ) {
				m_DontSwap_list.RemoveAt(pos);
				return false;
			}
			else
				return true;
		}
		else if (m_DontSwap_list.GetAt(pos).file == NULL) // in which cases should this happen?
			m_DontSwap_list.RemoveAt(pos);
	}

	return false;
}

// Maella -Extended clean-up II-
void CUpDownClient::CleanUp(CPartFile* pDeletedFile){
	// Check if all pointers to the delete file have been removed
	if(reqfile == pDeletedFile){
		ASSERT(FALSE);
		reqfile = NULL;
		AddPhoenixLogLine(false, _T("CleanUp() reports an error with reqfile"));
	}

	for(POSITION pos = m_OtherRequests_list.GetHeadPosition(); pos != 0; ){
		POSITION cur_pos = pos;
		CPartFile* cur_file = m_OtherRequests_list.GetNext(pos);
		if(cur_file == pDeletedFile){
			m_OtherRequests_list.RemoveAt(cur_pos);
			AddPhoenixLogLine(false, _T("CleanUp() reports an error with m_OtherRequests_list"));
		}
	}

	for(POSITION pos = m_OtherNoNeeded_list.GetHeadPosition(); pos != 0; ){
		POSITION cur_pos = pos;
		CPartFile* cur_file = m_OtherNoNeeded_list.GetNext(pos);
		if(cur_file == pDeletedFile){
			m_OtherNoNeeded_list.RemoveAt(cur_pos);
			AddPhoenixLogLine(false, _T("CleanUp() reports an error with m_OtherNoNeeded_list"));
		}
	}

	// [TPT] Maella -Auto A4AF-
	PartStatusMap::iterator it = m_partStatusMap.find(pDeletedFile); 
	if(it != m_partStatusMap.end()){
		m_partStatusMap.erase(it);
	}
	// Maella end
}
// Maella end

// Maella -Auto A4AF-
void CUpDownClient::TrigNextSafeAskForDownload(CPartFile* pFile){
	// Check when the specified file has been asked for the last time and
	// define when the file can be asked again without risking to be banished.
	if(pFile != NULL){
		PartStatusMap::const_iterator it = m_partStatusMap.find(pFile);
		if(it != m_partStatusMap.end()){
			// Compute then the next AskForDownload() might be 
			// performed without risk of Ban() (=> 11 minutes)
			if(it->second.dwStartUploadReqTime == 0){
				// File has never been asked before
				m_dwNextTCPAskedTime = 0; // Safe immediate reask
			}
			else {
				m_dwNextTCPAskedTime = it->second.dwStartUploadReqTime + MIN_REQUESTTIME + 60000;
			}
		}
		else {
			// File has never been asked before
			m_dwNextTCPAskedTime = 0; // Safe immediate reask
		}

		if(m_dwNextTCPAskedTime != 0 && m_dwNextTCPAskedTime > GetTickCount()){
			// Maella -Filter verbose messages-
			if(theApp.glob_prefs->GetBlockMaellaSpecificDebugMsg() == false){
				AddPhoenixLogLine(false, _T("Reask '%s' for '%s' delayed to %u seconds"),
												GetUserName(),
												pFile->GetFileName(), 
												(m_dwNextTCPAskedTime - GetTickCount()) / 1000);
			}
			// Maella end
		}
	}
}
// Maella end
