#include "StdAfx.h"
#include "updownclient.h"
#include "emule.h"
#include "uploadqueue.h"
#include "Clientlist.h"
#include "SearchList.h"
#include <zlib/zlib.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//<<< eWombat [SUI]
void CUpDownClient::GetSUIStateStr(CString &string)
{
	if (!credits)
	{
		string.Empty();
		return;
	}
	switch(credits->GetCurrentIdentState(GetIP()))
	{
	case IS_IDNEEDED:
		string=GetResString(IDS_SUI_IDNEEDED);
		return;
	case IS_IDENTIFIED:
		string=GetResString(IDS_SUI_IDENTIFIED);
		return;
	case IS_IDFAILED:
		string=GetResString(IDS_SUI_IDFAILED);
		return;
	case IS_IDBADGUY:
		string=GetResString(IDS_SUI_IDBADGUY);
		return;
	}
	string.Empty();
	return;
}

int CUpDownClient::GetSUISort()
{
	if (!credits)
		return 5;
	switch(credits->GetCurrentIdentState(GetIP()))
		{
		case IS_IDENTIFIED:
			return 4;
		case IS_IDNEEDED:
			return 3;
		case IS_IDBADGUY:
			return 2;
		case IS_IDFAILED:
			return 1;
		}
	return 5;
}

int CUpDownClient::GetBanSort(void)
{
	return IsBanned()+(10*GetSnafuReason())+(2*IsNotSUI())+(4*IsSUIFailed());
}

void CUpDownClient::GetBanStr(CString &string)
{
	if (IsSnafu())
		string=GetSnafuReasonStr();
	else if (IsNotSUI())
		string=_T("[S.U.I.]");
	else
		string=(IsBanned() ? GetResString(IDS_YES):GetResString(IDS_NO));
}
//<<< eWombat [SUI]

// <<< eWombat [SNAFU]
void CUpDownClient::UnSnafu(bool bCheckTrust) 
{
	if (bCheckTrust && m_nSnafuReason != SNAFU_ACT && m_nSnafuReason != SNAFU_HASHCHG)
		return;
	ResetTrust();
	m_nSnafuReason=SNAFU_NONE;
	m_nSnafuTimer=0;
	theApp.emuledlg->transferwnd.queuelistctrl.RefreshClient(this);
	theApp.emuledlg->transferwnd.clientlistctrl.RefreshClient(this);
}

void CUpDownClient::DoSnafu(eSnafuReason reason , bool bCheckTrust)
{
	if (!theApp.glob_prefs->GetAntiSnafu() || m_bSnafuExclude)
		return;
	if (IsSnafu())
		{
		if (m_nTrust > 0)
			m_nTrust--;
		if (reason==SNAFU_ACT || reason == SNAFU_HASHCHG || reason == SNAFU_HASH || reason==SNAFU_MYHASH)
			m_nSnafuReason=reason;
		return;
		}
	if (bCheckTrust && m_nTrust > 0)
		{
		m_nTrust--;
		return;
		}

	//If it is on Upload-queue remove it
	if(GetUploadState() == US_UPLOADING || GetUploadState() == US_WAITCALLBACK || GetUploadState() == US_CONNECTING )
		theApp.uploadqueue->RemoveFromUploadQueue(this,USR_SNAFU);

	m_nSnafuReason   = reason;
	m_nSnafuTimer	 = ::GetTickCount();
	theApp.clientlist->AddSnafuClient(this);
	theApp.emuledlg->transferwnd.queuelistctrl.RefreshClient(this);
	theApp.emuledlg->transferwnd.clientlistctrl.RefreshClient(this);

}

CString	CUpDownClient::GetSnafuReasonStr()
{
	if (!theApp.glob_prefs->GetAntiSnafu())
		return "";

	switch(m_nSnafuReason)
	{
	case SNAFU_WOMBAT:
		return "Bad Wombat";
	case SNAFU_FRIENDSHAREMOD:
		return "Friendshare Mod";
	case SNAFU_NAME:
	case SNAFU_ACTION:
	case SNAFU_OPCODE:
		return "suspect behaviour";
	case SNAFU_NICK:
		return "suspect nickname";
	case SNAFU_MODSTR:
		return "suspect client version";
	case SNAFU_CLIENT:
		return "eDonkey client with eMule property";
	case SNAFU_HASHCHG:
		return "hash changer";
	case SNAFU_MYHASH:
		return "hash bastard";
	case SNAFU_ACT:
		return "hash thief (act)";
	case SNAFU_HASH:
		return "hash thief";
	case SNAFU_GPL:
		return "no gpl-conform mod";
	default:
		return "shit happens";
	}
}
void CUpDownClient::SnafuCheckMod(LPCTSTR pMod)
{
	if (!pMod)
		return;

	if (stristrex(pMod,"mison")!=NULL ||stristrex(pMod,"evort")!=NULL ||stristrex(pMod,"booster")!=NULL ||
		stristrex(pMod,"elfen")!=NULL ||stristrex(pMod,"imp delta")!=NULL ||stristrex(pMod,"father")!=NULL)
		m_bGoofy=true;

	if (!theApp.glob_prefs->GetAntiSnafu())
		return;
	if (IsSnafu())
		return;
	//CString test=m_strMod;
	//test.MakeLower();
	if (m_bGoofy ||
		stristrex(pMod,"fincan")!=NULL||
		stristrex(pMod,"dodgethis")!=NULL|| //Updated
		stristrex(pMod,"freeza")!=NULL||
		stristrex(pMod,"d-unit")!=NULL||
		stristrex(pMod,"nos")!=NULL||
		stristrex(pMod,"imperator")!=NULL||
		stristrex(pMod,"speedload")!=NULL||
		stristrex(pMod,"gt mod")!=NULL||
		stristrex(pMod,"egomule")!=NULL||
		stristrex(pMod,"darkmule")!=NULL||
		stristrex(pMod,"legolas")!=NULL||
		stristrex(pMod,"dm-")!=NULL|| //hotfix
		stristrex(pMod,"|x|")!=NULL||
		stristrex(pMod,"esladevil")!=NULL||
		stristrex(pMod,"dragon")!=NULL)
	{	
		AddPhoenixLogLine("Detected: %s",pMod);
		DoSnafu(SNAFU_MODSTR);
	}
}

void CUpDownClient::SnafuCheckNick(LPCTSTR pNick)
{
	if (!theApp.glob_prefs->GetAntiSnafu())
		return;
	if (pNick==NULL)
		return;
	if (IsSnafu())
		return;
	if(	
		(stristrex(pNick,"emule") && stristrex(pNick,"booster")) ||
		stristrex(pNick,"pharao")		|| 
		stristrex(pNick,"reverse") 		||
		stristrex(pNick,"rammstein")	||
		stristrex(pNick,"leecha")		||
		stristrex(pNick,"energyfaker")	||
		stristrex(pNick,"edevil")		||
		stristrex(pNick,"darkmule")		||
		stristrex(pNick,"evortex")		||
		stristrex(pNick,"|evorte|x|")	||
		stristrex(pNick,"$gam3r$")		||
		stristrex(pNick,"g@m3r")		||
		stristrex(pNick,"$warez$")		||
		stristrex(pNick,"celinesexy"))
	{	
		AddPhoenixLogLine("Suspect nickname: %s",pNick);
		DoSnafu(SNAFU_NICK);
	}
}
void CUpDownClient::SnafuCheckVersion()
{
	if (IsSnafu())
		return;
	if ((GetVersion()>589) && ((GetSourceExchangeVersion()>0) || (GetExtendedRequestsVersion()>0)) && (GetClientSoft()==SO_EDONKEY)) //Fake Donkeys
	{	
		DoSnafu(SNAFU_CLIENT);
	}
	if ( (GetClientSoft() == SO_EDONKEYHYBRID) &&  ( (GetSourceExchangeVersion()>0) || (GetExtendedRequestsVersion()>0))) //Fake Hybrids
	{
		DoSnafu(SNAFU_CLIENT);
	}
	if (m_bySupportSecIdent!=0 && (GetClientSoft()==SO_EDONKEYHYBRID || GetClientSoft()==SO_EDONKEY))
	{
		DoSnafu(SNAFU_CLIENT);
	}
}

ULONG CUpDownClient::SnafuNameCRC(char* tocalc)
{   
	return theApp.wombatlist->GetHashKey(tocalc);
}

//>>> eWombat [ACT]

// >>> eWombat [SNAFU]
#ifdef _DEBUGWOMBAT
void CUpDownClient::GetDebugStr(CString &string)
{
string=m_strCommunity;
}
#endif

void  CUpDownClient::UnBan()
{
	theApp.clientlist->AddTrackClient(this); // [TPT] - Patch	
	SetUploadState(US_NONE);

	ClearWaitStartTime();

	for (POSITION pos = m_RequestedFiles_list.GetHeadPosition();pos != 0;m_RequestedFiles_list.GetNext(pos))
	{
		Requested_File_Struct* cur_struct = m_RequestedFiles_list.GetAt(pos);
		cur_struct->badrequests = 0;
		cur_struct->lastasked = 0;	
	}
	m_nBanTimer = 0;
	m_bBan		= false;
	theApp.emuledlg->transferwnd.queuelistctrl.RefreshClient(this);
	theApp.emuledlg->transferwnd.clientlistctrl.RefreshClient(this);
}

bool CUpDownClient::IsBanned()
{	
	return( m_bBan && (m_nDownloadState != DS_DOWNLOADING));
}

void CUpDownClient::Ban()
{
	theApp.clientlist->AddTrackClient(this); // [TPT] - Patch
	if ( !IsBanned())
		theApp.emuledlg->AddDebugLogLine(false,GetResString(IDS_CLIENTBLOCKED),GetUserName());
	m_nBanTimer = ::GetTickCount();
	m_bBan		= true;
	theApp.clientlist->AddSnafuClient(this);	
	SetUploadState(US_BANNED); // [TPT] - Patch
	theApp.emuledlg->transferwnd.queuelistctrl.RefreshClient(this);
	theApp.emuledlg->transferwnd.clientlistctrl.RefreshClient(this);
}
