//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include <zlib/zlib.h>
#include "UpDownClient.h"
#include "opcodes.h"
#include "packets.h"
#include "emule.h"
#include "uploadqueue.h"
#include "otherstructs.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//	members of CUpDownClient
//	which are mainly used for uploading functions 

// [TPT]
// Maella -Code Improvement-
void CUpDownClient::DrawUpStatusBar(CDC* dc, RECT* rect, bool onlygreyrect, bool  bFlat){ 
	const COLORREF crBoth = (bFlat) ? RGB(0, 0, 0) : RGB(104, 104, 104); 
	const COLORREF crNeither = (bFlat) ? RGB(224, 224, 224) : RGB(224, 224, 224);
	const COLORREF crClientOnly = (bFlat) ? RGB(0, 220, 255) : RGB(0, 220, 255); 
	const COLORREF crSending = (bFlat) ? RGB(0,150,0) : RGB(0, 150, 0); 
	const COLORREF crNextSending = (bFlat) ? RGB(255,208,0) : RGB(255,208,0); 

    // wistily: UpStatusFix
	CKnownFile* currequpfile = theApp.sharedfiles->GetFileByID(requpfileid);
	uint32 filesize;
	if (currequpfile)
		filesize=currequpfile->GetFileSize();
	else
		filesize=PARTSIZE*(m_nUpPartCount);
	// wistily: UpStatusFix
	
	// Set size and fill with default color (grey)
	CBarShader statusBar(rect->bottom - rect->top, rect->right - rect->left);
	statusBar.SetFileSize(filesize); 
	statusBar.Fill(crNeither); 

	if (!onlygreyrect && m_abyUpPartStatus) { 
		for (uint32 i = 0;i != m_nUpPartCount;i++)
			if(m_abyUpPartStatus[i])
				statusBar.FillRange(PARTSIZE*(i),PARTSIZE*(i+1),crBoth);
	}
	Requested_Block_Struct* block;
	if (!m_BlockRequests_queue.IsEmpty()){
		block = m_BlockRequests_queue.GetHead();
		if(block){
			uint32 start = block->StartOffset/PARTSIZE;
			statusBar.FillRange(start*PARTSIZE, (start+1)*PARTSIZE, crNextSending);
		}
	}
	if (!m_DoneBlocks_list.IsEmpty()){
		block = m_DoneBlocks_list.GetTail();
		if(block){
			uint32 start = block->StartOffset/PARTSIZE;
			statusBar.FillRange(start*PARTSIZE, (start+1)*PARTSIZE, crNextSending);
		}
	}
	if (!m_DoneBlocks_list.IsEmpty()){
		for(POSITION pos=m_DoneBlocks_list.GetHeadPosition();pos!=0;m_DoneBlocks_list.GetNext(pos)){
			Requested_Block_Struct* block = m_DoneBlocks_list.GetAt(pos);
			statusBar.FillRange(block->StartOffset, block->EndOffset, crSending);
		}
	}
   	statusBar.Draw(dc, rect->left, rect->top, bFlat); 
} 
// Maella end 

// Maella -Code Improvement-
void CUpDownClient::SetUploadState(EUploadState news){
	if(m_nUploadState != news){

		// Maella -Accurate measure of bandwidth: eDonkey data + control, network adapter-
		if(m_nUploadState == US_UPLOADING || news == US_UPLOADING){
			m_nUpDatarate = 0;
			m_nUpDatarateMeasure = 0;
			m_upHistory_list.RemoveAll();

			// Maella -New bandwidth control-
			if(news == US_UPLOADING){
				// Reduce the size of the buffer => avoid latency caused by buffer
				int oldValue = 0;
				int newValue = theApp.glob_prefs->GetSendSocketBufferSize();
				int size = sizeof(oldValue);
				if(socket != NULL &&
				   socket->GetSockOpt(SO_SNDBUF, &oldValue, &size, SOL_SOCKET) == TRUE &&
				   socket->SetSockOpt(SO_SNDBUF, &newValue, sizeof(newValue), SOL_SOCKET) == TRUE){
					// Maella -Filter verbose messages-
					if(theApp.glob_prefs->GetBlockMaellaSpecificDebugMsg() == false){
						AddPhoenixLogLine(false, _T("Change size from %u to %u of the sending buffer"), oldValue, newValue);
					}
					// Maella end
				}
				else {
					AddDebugLogLine(false, _T("Failure to change size from %u to %u of the sending buffer"), oldValue, newValue);
				}
			}
			// Maella end
		}
		// Maella end

	m_nUploadState = news;
	theApp.emuledlg->transferwnd.clientlistctrl.RefreshClient(this);
	}
}
// Maella end

// [TPT]
uint32 CUpDownClient::GetScore(bool sysvalue, bool isdownloading, bool onlybasevalue){
	
	// Maella -Code Improvement-
	if (m_pszUsername == NULL || GetUploadFileID() == NULL)	
		return 0;

	if (credits == 0){
		ASSERT ( false );
		return 0;
	}
	CKnownFile* currequpfile = theApp.sharedfiles->GetFileByID(requpfileid);
	if(!currequpfile)
		return 0;
	// Maella end
	
	// bad clients (see note in function)
	if (credits->GetCurrentIdentState(GetIP()) == IS_IDBADGUY)
		return 0;
	// friend slot
	if (IsFriend() && GetFriendSlot() && !HasLowID())
		return 0x0FFFFFFF;

	if (IsSnafu() || IsNotSUI() || IsBanned())//eWombat [SNAFU][TPT]
		return 0;

	if (sysvalue && HasLowID() && !(socket && socket->IsConnected())){
		return 0;
	}

    // Maella -One-queue-per-file- (idea bloodymad)
	if(theApp.glob_prefs->GetEnableMultiQueue() == false){

	// TODO coded by tecxx & herbert, one yet unsolved problem here:
	// sometimes a client asks for 2 files and there is no way to decide, which file the 
	// client finally gets. so it could happen that he is queued first because of a 
	// high prio file, but then asks for something completely different.
	int filepriority = 10; // standard
		switch(currequpfile->GetUpPriority()){ 
				case PR_VERYHIGH:
					filepriority = 18;
					break;
				case PR_HIGH: 
					filepriority = 9; 
					break; 
				case PR_LOW: 
					filepriority = 6; 
					break; 
				case PR_VERYLOW:
					filepriority = 2;
					break;
				case PR_NORMAL: 
					default: 
					filepriority = 7; 
				break; 
			} 

	// calculate score, based on waitingtime and other factors
	float fBaseValue;
	if (onlybasevalue)
		fBaseValue = 100;
	else if (!isdownloading)
			fBaseValue = (float)(::GetTickCount()-GetWaitStartTime())/1000.0f;
	else{
		// we dont want one client to download forever
		// the first 15 min downloadtime counts as 15 min waitingtime and you get a 15 min bonus while you are in the first 15 min :)
		// (to avoid 20 sec downloads) after this the score won't raise anymore 
		fBaseValue = (float)(m_dwUploadTime-GetWaitStartTime());
		ASSERT ( m_dwUploadTime-GetWaitStartTime() >= 0 ); //oct 28, 02: changed this from "> 0" to ">= 0"
			fBaseValue += (::GetTickCount() - m_dwUploadTime > 900000) ? 900000.0f : 1800000.0f;
			fBaseValue /= 1000.0f;
	}
	if(theApp.glob_prefs->UseCreditSystem())
	{
		float modif = credits->GetScoreRatio(GetIP());
		fBaseValue *= modif;
		if( !m_bySupportSecIdent && modif == 1 )
			fBaseValue *= 0.85f;
	}
	if (!onlybasevalue)
		fBaseValue *= (float(filepriority)/10.0f); 
		// Already tested above
		//if (!isdownloading && !onlybasevalue){
		//	if (sysvalue && HasLowID() && !(socket && socket->IsConnected()) ){
		//		if (!theApp.serverconnect->IsConnected() || theApp.serverconnect->IsLowID() || theApp.listensocket->TooManySockets())
		//			return 0;
		//	}
		//}
	if( (IsEmuleClient() || this->GetClientSoft() < 10) && m_byEmuleVersion <= 0x19 )
		fBaseValue *= 0.5f;

	return (uint32)fBaseValue;
	}
	else {
		if(onlybasevalue == true){
			// Default value
			float fBaseValue = 100.0f;

			// Bonus Credit
			if(theApp.glob_prefs->UseCreditSystem() == true && credits != NULL)
				fBaseValue *= credits->GetScoreRatio(GetIP());
			
			// Penality client types
			if((IsEmuleClient() || this->GetClientSoft() < 10) && m_byEmuleVersion <= 0x19)
				fBaseValue *= 0.5f;
	
	return (uint32)fBaseValue;
		}
		else {
			// Client score
			// Remark: there is an overflow after ~49 days
			uint32 clientScore;
			if(isdownloading == false){
				clientScore = ::GetTickCount()-GetWaitStartTime();
			}
			else {
				// we dont want one client to download forever
				// the first 15 min downloadtime counts as 15 min waitingtime and you get a 15 min bonus while you are in the first 15 min :)
				// (to avoid 20 sec downloads) after this the score won't raise anymore
				clientScore = (m_dwUploadTime-GetWaitStartTime());
				clientScore += (::GetTickCount() - m_dwUploadTime > 900000) ? 900000 : 1800000;
			}
			clientScore /= 1000; // about +1 point each second

			// Bonus Credit
			if(theApp.glob_prefs->UseCreditSystem() == true && credits != NULL)
				clientScore = (uint32)(credits->GetScoreRatio(GetIP()) * clientScore);

			// Penality client types
			if((IsEmuleClient() || this->GetClientSoft() < 10) && m_byEmuleVersion <= 0x19)
				clientScore /= 2;

			// File Score
			// Remark: there is an overflow after ~49 days
			uint32 fileScore = currequpfile->GetFileScore(isdownloading, GetUpStartTimeDelay()); // about +1 point each second 

			// Final score	
			// Remark: The whole timming of eMule should be rewritten. 
			//         The rollover of the main timer (GetTickCount) is not supported.
			//         A logarithmic scale would fit better here, but it might be slower.
			uint32 runTime = (GetTickCount() - theApp.stat_starttime) / 1000;
			if(runTime <= 3600)
				// Less than 1 hour
				return 1000*fileScore + clientScore; // 1 second resolution
			if(runTime <= 2*3600)
				// Less than 2 hours
				return 1000*fileScore + clientScore/10; // 10 seconds resolution
			else
				// More than 2 hours
				return 1000*fileScore + clientScore/100; // 100 seconds resolution
		}
	}
	// Maella end
}

// Checks if it is next requested block from another chunk of the actual file or from another file 
// 
// [Returns] 
//   true : Next requested block is from another different chunk or file than last downloaded block 
//   false: Next requested block is from same chunk that last downloaded block 
bool CUpDownClient::IsDifferentPartBlock()
{ 
	Requested_Block_Struct* lastBlock;
	Requested_Block_Struct* currBlock;
	uint32 lastDone = 0;
	uint32 currRequested = 0;
	
	bool different = false;
	
	try {
		// Check if we have good lists and proceed to check for different chunks
		if (!m_BlockRequests_queue.IsEmpty() && !m_DoneBlocks_list.IsEmpty())
		{
			// Calculate corresponding parts to blocks
			lastBlock = (Requested_Block_Struct*)m_DoneBlocks_list.GetTail();
			lastDone = lastBlock->StartOffset / PARTSIZE;
			currBlock = (Requested_Block_Struct*)m_BlockRequests_queue.GetHead(); 
			currRequested = currBlock->StartOffset / PARTSIZE; 
             
			// Test is we are asking same file and same part
			if ( lastDone != currRequested)
			{ 
				different = true;
				//AddDebugLogLine(false, "%s: Upload session ended due to new chunk.", this->GetUserName());
			}
			if (md4cmp(lastBlock->FileID, currBlock->FileID) != 0)
			{ 
				different = true;
				// Maella -Filter verbose messages-
				if(theApp.glob_prefs->GetBlockUploadEndMsg() == false){
					AddPhoenixLogLine(false, "%s: Upload session ended due to different file.", this->GetUserName());
			}
				// Maella end
			}
		} 
   	}
   	catch(...)
   	{ 
			AddDebugLogLine(false, "%s: Upload session ended due to error.", this->GetUserName());
      		different = true; 
   	} 
//	AddDebugLogLine(false, "Debug: User %s, last_done_part (%u) %s (%u) next_requested_part, sent %u Kbs.", GetUserName(), last_done_part, different_part? "!=": "==", next_requested_part, this->GetTransferedUp() / 1024); 

	return different; 
}

class CSyncHelper
{
public:
	CSyncHelper()
	{
		m_pObject = NULL;
	}
	~CSyncHelper()
	{
		if (m_pObject)
			m_pObject->Unlock();
	}
	CSyncObject* m_pObject;
};

// [TPT]
bool CUpDownClient::CreateNextBlockPackage(){
	// Maella -Upload Stop Reason-
	// Maella -Allow Hybrid to download from eMule-
	if (theApp.uploadqueue->CheckForTimeOver(this) == true){
		// TimeOver => back on the waitqueue
		theApp.uploadqueue->RemoveFromUploadQueue(this, CUpDownClient::USR_TIMEOVER);
				theApp.uploadqueue->AddClientToQueue(this,true);

		// Signal to the remote side the end of the upload session
		if(socket != NULL){
			Packet* packet = new Packet(OP_OUTOFPARTREQS, 0);
			theApp.uploadqueue->AddUpDataOverheadOther(packet->size);
			socket->SendPacket(packet, true, true);
			}

		return false;
		}
	else
	if ( !GetFriendSlot() ){
		if(theApp.glob_prefs->TransferFullChunks() == true && IsDifferentPartBlock() == true){
			// Different chunk => back on the waitqueue
			theApp.uploadqueue->RemoveFromUploadQueue(this, CUpDownClient::USR_NEW_CHUNK);
				theApp.uploadqueue->AddClientToQueue(this,true);

			// Signal to the remote side the end of the upload session
			if(socket != NULL){
				Packet* packet = new Packet(OP_OUTOFPARTREQS, 0);
				theApp.uploadqueue->AddUpDataOverheadOther(packet->size);
				socket->SendPacket(packet, true, true);
			}
		
			return false;
		}
	}
	// Maella end
	if(m_BlockRequests_queue.IsEmpty()){
		return false;
	}
	CFile file;
	byte* filedata = 0;
	CString fullname;
	// -khaos--+++> Statistic to breakdown uploaded data by complete file vs. partfile.
	bool bFromPF = true;
	// <-----khaos-
	CSyncHelper lockFile;
	try{
		while (!m_BlockRequests_queue.IsEmpty()){
			Requested_Block_Struct* currentblock = m_BlockRequests_queue.GetHead();
			CKnownFile* srcfile = theApp.sharedfiles->GetFileByID(currentblock->FileID);
			if (!srcfile)
				throw GetResString(IDS_ERR_REQ_FNF);

			if (srcfile->IsPartFile() && ((CPartFile*)srcfile)->GetStatus() != PS_COMPLETE){
				// Do not access a part file, if it is currently moved into the incoming directory.
				// Because the moving of part file into the incoming directory may take a noticable 
				// amount of time, we can not wait for 'm_FileCompleteMutex' and block the main thread.
				if (!((CPartFile*)srcfile)->m_FileCompleteMutex.Lock(0)){ // just do a quick test of the mutex's state and return if it's locked.
					return false;
				}
				lockFile.m_pObject = &((CPartFile*)srcfile)->m_FileCompleteMutex;
				// If it's a part file which we are uploading the file remains locked until we've read the
				// current block. This way the file completion thread can not (try to) "move" the file into
				// the incoming directory.

				fullname = RemoveFileExtension(((CPartFile*)srcfile)->GetFullName());
			}
			else{
				fullname.Format("%s\\%s",srcfile->GetPath(),srcfile->GetFileName());
			}
		
			uint32 togo;
			if (currentblock->StartOffset > currentblock->EndOffset){
				togo = currentblock->EndOffset + (srcfile->GetFileSize() - currentblock->StartOffset);
			}
			else{
				togo = currentblock->EndOffset - currentblock->StartOffset;
				if (srcfile->IsPartFile() && !((CPartFile*)srcfile)->IsComplete(currentblock->StartOffset,currentblock->EndOffset-1))
					throw GetResString(IDS_ERR_INCOMPLETEBLOCK);
			}
		
			// [TPT]
			// Maella -Allow Hybrid to download from eMule-	
			if(togo > EMBLOCKSIZE*3 && GetClientSoft() != SO_EDONKEYHYBRID)
				throw GetResString(IDS_ERR_LARGEREQBLOCK);
			else if(togo > 243200)
				// Only Hybrid is allowed to request larger blocks, otherwise 
				// it could be exploited by leechers to stay longer in the 
				// upload queue (see when CUploadQueue::CheckForTimeOver() is called)
				throw GetResString(IDS_ERR_LARGEREQBLOCK);
			// Maella end

			if (!srcfile->IsPartFile()){
				// -khaos--+++> This is not a part file...
				bFromPF = false;
				// <-----khaos-
				if (!file.Open(fullname,CFile::modeRead|CFile::osSequentialScan|CFile::shareDenyNone))
					throw GetResString(IDS_ERR_OPEN);
				file.Seek(currentblock->StartOffset,0);
				
				filedata = new byte[togo+500];
				if (uint32 done = file.Read(filedata,togo) != togo){
					file.SeekToBegin();
					file.Read(filedata + done,togo-done);
				}
				file.Close();
			}
			else{
				CPartFile* partfile = (CPartFile*)srcfile;

				partfile->m_hpartfile.Seek(currentblock->StartOffset,0);
				
				filedata = new byte[togo+500];
				if (uint32 done = partfile->m_hpartfile.Read(filedata,togo) != togo){
					partfile->m_hpartfile.SeekToBegin();
					partfile->m_hpartfile.Read(filedata + done,togo-done);
				}
			}
			if (lockFile.m_pObject){
				lockFile.m_pObject->Unlock(); // Unlock the (part) file as soon as we are done with accessing it.
				lockFile.m_pObject = NULL;
			}

			SetUploadFileID(currentblock->FileID);

			// [TPT]
			// Maella -Check for file name extension- fix for lower/upper case in filename
			if(m_byDataCompVer == 1 && srcfile->IsArchive() == false){
				// Compression is supported by the remote client
				CreatePackedPackets(filedata,togo,currentblock,bFromPF);
			}
			else {
				// Remote client doesn't support compression or
				// File already compressed. Don't waste CPU time to recompress it.
				CreateStandartPackets(filedata,togo,currentblock,bFromPF);
			}
			// Maella end
			
			// file statistic
			srcfile->statistic.AddTransferred(currentblock->StartOffset, togo);	// [TPT] - SLUGFILLER: Spreadbars

			m_DoneBlocks_list.AddHead(m_BlockRequests_queue.RemoveHead());
			delete[] filedata;
			filedata = 0;
			// [TPT]
			// Maella -One-queue-per-file- (idea bloodymad)
			srcfile->UpdateStartUploadTime();
			// Maella end
		}
	}
	catch(CString error){
		OUTPUT_DEBUG_TRACE();
		// [TPT]
		// Maella -Code Improvement-
		CString fullName; 
		fullName.Format(_T("%s (client=%s)"), GetUserName(), GetClientSoftFullName());
		AddDebugLogLine(false,GetResString(IDS_ERR_CLIENTERRORED), fullName, error);
		// Maella end
		theApp.uploadqueue->RemoveFromUploadQueue(this, CUpDownClient::USR_EXCEPTION); // Maella -Upload Stop Reason-
		SetUploadFileID(NULL);
		if (filedata)
			delete[] filedata;
		return false;
	}
	catch(CFileException* e){
		TCHAR szError[MAX_CFEXP_ERRORMSG];
		e->GetErrorMessage(szError, ARRSIZE(szError));
		// Maella -Code Improvement-
		AddDebugLogLine(false,_T("Failed to create upload package for %s (client=%s) - %s"), GetUserName(), GetClientSoftFullName(), szError);
		// Maella end
		theApp.uploadqueue->RemoveFromUploadQueue(this, CUpDownClient::USR_EXCEPTION);
		SetUploadFileID(NULL);
		if (filedata)
			delete[] filedata;
		e->Delete();
		return false;
	}
//	AddDebugLogLine(false,"Debug: Packet done. Size: %i",blockpack->GetLength());
	return true;
}

void CUpDownClient::ProcessUpFileStatus(char* packet,uint32 size){
	if (m_abyUpPartStatus) {
		delete[] m_abyUpPartStatus;
		m_abyUpPartStatus = NULL;	// added by jicxicmic
	}
	m_nUpPartCount = 0;
	m_nUpCompleteSourcesCount= 0;
	if( size == 16 )
		return;
	CSafeMemFile data((BYTE*)packet,size);
	uchar cfilehash[16];
	data.Read(cfilehash,16);
	CKnownFile* tempreqfile = theApp.sharedfiles->GetFileByID(cfilehash);
	uint16 nED2KUpPartCount;
	data.Read(&nED2KUpPartCount,2);
	if (!nED2KUpPartCount){
		m_nUpPartCount = tempreqfile->GetPartCount();
		m_abyUpPartStatus = new uint8[m_nUpPartCount];
		MEMSET(m_abyUpPartStatus,0,m_nUpPartCount);
	}

	else{
		if (tempreqfile->GetED2KPartCount() != nED2KUpPartCount){
			m_nUpPartCount = 0;
			return;
		}
		m_nUpPartCount = tempreqfile->GetPartCount();
		m_abyUpPartStatus = new uint8[m_nUpPartCount];
		uint16 done = 0;
		while (done != m_nUpPartCount){
			uint8 toread;
			data.Read(&toread,1);
			for (sint32 i = 0;i != 8;i++){
				m_abyUpPartStatus[done] = ((toread>>i)&1)? 1:0;
//				We may want to use this for another feature..
//				if (m_abyUpPartStatus[done] && !tempreqfile->IsComplete(done*PARTSIZE,((done+1)*PARTSIZE)-1))
//					bPartsNeeded = true;
				done++;
				if (done == m_nUpPartCount)
					break;
			}
		}
		if (/*(GetExtendedRequestsVersion() > 1) && */(data.GetLength() - data.GetPosition() > 1))
		{
			uint16 nCount;
			data.Read(&nCount,2);
			SetUpCompleteSourcesCount(nCount);
		}
	}
	
	// //[TPT] - Official patch. MORPH START - Changed by SiRoB, HotFix Due to Complete Source Feature
	//tempreqfile->NewAvailPartsInfo();
	if(!tempreqfile->IsPartFile())
		tempreqfile->NewAvailPartsInfo();
	else
		((CPartFile*)tempreqfile)->NewSrcPartsInfo();
	//MORPH END  - Changed by SiRoB, HotFix Due to Complete Source Feature

	theApp.emuledlg->transferwnd.queuelistctrl.RefreshClient(this);
	// [TPT]
	// Maella -Code Improvement-
	if(tempreqfile->IsPartFile() == true && m_nUpPartCount != 0){
		// Check if a source has now chunk that we can need
		POSITION pos = m_OtherNoNeeded_list.Find(tempreqfile);
		if(pos != NULL){
			for(uint16 i = 0; i < m_nUpPartCount; i++){ 
				if(m_abyUpPartStatus[i] != 0){ 
					const uint32 uStart = PARTSIZE*i;
					const uint32 uEnd = (tempreqfile->GetFileSize()-1 <= (uStart+PARTSIZE-1)) ? (tempreqfile->GetFileSize()-1) : (uStart+PARTSIZE-1);
					if(((CPartFile*)tempreqfile)->IsComplete(uStart, uEnd) == false){
						// Swap source to the other list
						m_OtherNoNeeded_list.RemoveAt(pos);
                        m_OtherRequests_list.AddHead((CPartFile*)tempreqfile);

						AddDebugLogLine(false, _T("Source '%s' has some part(s) available now for '%s'"), 
														 GetUserName(),
														 tempreqfile->GetFileName());					
						break; // [TPT] - Xman Bugfix
					}
				}
			}
		}	
	}
	// Maella end
}

// -khaos--+++> Added new parameter: bool bFromPF
void CUpDownClient::CreateStandartPackets(byte* data,uint32 togo, Requested_Block_Struct* currentblock, bool bFromPF){
	// <-----khaos-
	uint32 nPacketSize;
	CMemFile memfile((BYTE*)data,togo);
	if (togo > 10240) 
		nPacketSize = togo/(uint32)(togo/10240);
	else
		nPacketSize = togo;
	while (togo){
		if (togo < nPacketSize*2)
			nPacketSize = togo;
		togo -= nPacketSize;
		// -khaos--+++> Create the packet with the new boolean.
		Packet* packet = new Packet(OP_SENDINGPART,nPacketSize+24, OP_EDONKEYPROT, bFromPF);
		// <-----khaos-
		md4cpy(&packet->pBuffer[0],GetUploadFileID());
		uint32 statpos = (currentblock->EndOffset - togo) - nPacketSize;
		MEMCOPY(&packet->pBuffer[16],&statpos,4);
		uint32 endpos = (currentblock->EndOffset - togo);
		MEMCOPY(&packet->pBuffer[20],&endpos,4);
		memfile.Read(&packet->pBuffer[24],nPacketSize);
		m_BlockSend_queue.AddTail(packet);
	}
}

// -khaos--+++> Added new parameter: bool bFromPF
void CUpDownClient::CreatePackedPackets(byte* data,uint32 togo, Requested_Block_Struct* currentblock, bool bFromPF){
	// <-----khaos-
	BYTE* output = new BYTE[togo+300];
	uLongf newsize = togo+300;
	uint16 result = compress2(output,&newsize,data,togo,9);
	if (result != Z_OK || togo <= newsize){
		delete[] output;
		// -khaos--+++>  Our new boolean...
		CreateStandartPackets(data,togo,currentblock,bFromPF);
		// <-----khaos-
		return;
	}
	m_bUsedComprUp = true;
	CMemFile memfile(output,newsize);
	togo = newsize;
	uint32 nPacketSize;
	if (togo > 10240) 
		nPacketSize = togo/(uint32)(togo/10240);
	else
		nPacketSize = togo;
	while (togo){
		if (togo < nPacketSize*2)
			nPacketSize = togo;
		togo -= nPacketSize;
		// -khaos--+++> Create the packet with the new boolean.
		Packet* packet = new Packet(OP_COMPRESSEDPART,nPacketSize+24,OP_EMULEPROT,bFromPF);
		// <-----khaos-
		md4cpy(&packet->pBuffer[0],GetUploadFileID());
		uint32 statpos = currentblock->StartOffset;
		MEMCOPY(&packet->pBuffer[16],&statpos,4);
		MEMCOPY(&packet->pBuffer[20],&newsize,4);
		memfile.Read(&packet->pBuffer[24],nPacketSize);
		m_BlockSend_queue.AddTail(packet);
	}
	delete[] output;
}

void CUpDownClient::SetUploadFileID(uchar* tempreqfileid){
	CKnownFile* newreqfile = NULL;
	if( tempreqfileid )
		newreqfile = theApp.sharedfiles->GetFileByID(tempreqfileid);
	CKnownFile* oldreqfile = theApp.sharedfiles->GetFileByID(requpfileid);
	if(newreqfile == oldreqfile)
		return;
	if(newreqfile){
		newreqfile->AddQueuedCount();
		newreqfile->AddUploadingClient(this);
		md4cpy(requpfileid,tempreqfileid);
	}
	else{
		md4clr(requpfileid);
	}
	if(oldreqfile){
		oldreqfile->SubQueuedCount();
		oldreqfile->RemoveUploadingClient(this);
	}
}

// [TPT]
// Maella -New bandwidth control-
void CUpDownClient::SplitHeadPacket(){

	// Maella -MTU Configuration-
	const uint32 MaxFragSize = theApp.glob_prefs->GetMTU() - (20 /*IP*/ + 20 /*TCP*/);
	// Maella end

	Packet* pHeadPacket = m_BlockSend_queue.GetHead();
	if(pHeadPacket != NULL && pHeadPacket->IsSplitted() == false && pHeadPacket->GetRealPacketSize() > MaxFragSize){
		// -khaos--+++> When splitting packets, we need to maintain bFromPF, otherwise...
		//				the stats get distorted.
		bool bFromPF = pHeadPacket->IsFromPF();
		// <-----khaos-

		// Split packet
		const uint32 nSize = pHeadPacket->GetRealPacketSize();
		const char* pBuffer = pHeadPacket->DetachPacket();

		// Remove packet from list to split it
		delete m_BlockSend_queue.RemoveHead();
		
		// Average size of splited packet
		const uint32 numberPacket = (nSize + (MaxFragSize - 1)) / MaxFragSize;
		const uint32 sizeSplitedPacket = nSize / numberPacket;

		// Create n packets of the same size (shall improve the ping)
		bool bLast = true;
		uint32 nPos = nSize;
		while(nPos > 0){
			uint32 nNewSize = (nPos < sizeSplitedPacket) ? nPos : sizeSplitedPacket;				
			nPos -= nNewSize;
			char* pBuffer2 = new char[nNewSize];
			MEMCOPY(pBuffer2, pBuffer+nPos, nNewSize);
			// -khaos--+++> Include bFromPF in our new packets to maintain statistical integrity.
			m_BlockSend_queue.AddHead(new Packet(pBuffer2, nNewSize, bLast, bFromPF));
			// <-----khaos-
			bLast = false;
		}

		delete[] pBuffer;
	}
}
// Maella end

void CUpDownClient::AddReqBlock(Requested_Block_Struct* reqblock){
	// [TPT] - SLUGFILLER: reqBlocksClipping
	CKnownFile* currequpfile = theApp.sharedfiles->GetFileByID(reqblock->FileID);
	if (!currequpfile) {
		delete reqblock;
		return;
	}
	uint32 filesize = currequpfile->GetFileSize();
	// Handle file wrap-around
	reqblock->StartOffset %= filesize;
	reqblock->EndOffset %= filesize;
	if (!reqblock->EndOffset)
		reqblock->EndOffset = filesize;
	if (reqblock->StartOffset == reqblock->EndOffset) {
		delete reqblock;
		return;
	}
	if (reqblock->EndOffset < reqblock->StartOffset) {
		Requested_Block_Struct* block = new Requested_Block_Struct;
		block->StartOffset = reqblock->StartOffset;
		block->EndOffset = filesize;
		md4cpy(block->FileID, reqblock->FileID);
		AddReqBlock(block);
		reqblock->StartOffset = 0;
	}
	// Handle part overflows and block-size overflows
	while(true) {
		uint32 next = (reqblock->StartOffset/PARTSIZE+1)*PARTSIZE;
		if (next > reqblock->StartOffset+184320)
			next = reqblock->StartOffset+184320;
		if (next < reqblock->EndOffset) {
			Requested_Block_Struct* block = new Requested_Block_Struct;
			block->StartOffset = reqblock->StartOffset;
			block->EndOffset = next;
			md4cpy(block->FileID, reqblock->FileID);
			AddReqBlock(block);
			reqblock->StartOffset = next;
			continue;
		}
		break;
	}
	// [TPT] - SLUGFILLER: reqBlocksClipping

	for (POSITION pos = m_DoneBlocks_list.GetHeadPosition();pos != 0;m_DoneBlocks_list.GetNext(pos)){
		if (reqblock->StartOffset == m_DoneBlocks_list.GetAt(pos)->StartOffset && reqblock->EndOffset == m_DoneBlocks_list.GetAt(pos)->EndOffset){
			delete reqblock;
			return;
		}
	}
	for (POSITION pos = m_BlockRequests_queue.GetHeadPosition();pos != 0;m_BlockRequests_queue.GetNext(pos)){
		if (reqblock->StartOffset == m_BlockRequests_queue.GetAt(pos)->StartOffset && reqblock->EndOffset == m_BlockRequests_queue.GetAt(pos)->EndOffset){
			delete reqblock;
			return;
		}
	}
	m_BlockRequests_queue.AddTail(reqblock);

}

// [TPT]
// Maella -New bandwidth control-	
uint32 CUpDownClient::SendBlockData(uint32 nMaxAmmount){

	uint32 nMaxAmmountUsed = 0;
	
	if(socket && socket->IsBusy() == false){

		if(m_BlockSend_queue.IsEmpty() == TRUE){
			if(CreateNextBlockPackage() == false) return 0;
		}

		// Splitting packets if necessary
		// Remark: - don't use 2*MAXFRAGSIZE (see CEMSocket::Send()), otherwise the 
		//           counter m_nMaxSendAllowed will trigged the sending of two TCP packets
		//           of 1300 (2*1300) at the same time => increase the ping of the application.
		if(m_BlockSend_queue.GetHead()->IsSplitted() == false && 
		   m_BlockSend_queue.GetHead()->GetRealPacketSize() > (uint32)(theApp.glob_prefs->GetMTU() - (20 /*IP*/ + 20 /*TCP*/))){
			SplitHeadPacket();
		}

		// Only one packet at the time may be sent
		if((socket->IsBusy() == false) &&
		   (m_BlockSend_queue.IsEmpty() == FALSE) &&
		   (nMaxAmmount >= m_BlockSend_queue.GetHead()->GetRealPacketSize())){
			
			Packet* tosend = m_BlockSend_queue.RemoveHead();
			uint32 nPacketSize = tosend->GetRealPacketSize();
			bool isHead = (tosend->IsSplitted() == false || tosend->IsLastSplitted() == true);

			// -khaos--+++>
			// Extended statistics information based on which client software and which port we sent this data to...
			// This also updates the grand total for sent bytes, etc.  And where this data came from.  Yeesh.
			theApp.glob_prefs->Add2SessionTransferData(GetClientSoft(), GetUserPort(), tosend->IsFromPF(), true, nPacketSize);

			sint32 result = socket->SendPacket(tosend, true, false, true);
			m_dwLastBlockSent = ::GetTickCount();//[TPT]-Cumulate Bandwidth
			// edonkey header should not be added to the credit
			if(result > 0){
				nMaxAmmount -= nPacketSize; // edonkey header not compensated here
				nMaxAmmountUsed += nPacketSize; // edonkey header not compensated here
			}
		}
	}

	return nMaxAmmountUsed;
}
// Maella end
	
// Maella -Accurate measure of bandwidth: eDonkey data + control, network adapter-
uint32 CUpDownClient::GetUploadDatarate(uint32 samples) const {
	uint32 nUpDatarate = 0;
    if(m_upHistory_list.GetSize() > 1 && samples >= 1){		// [TPT] - Xman Bugfix
		// Retieve the location of the n previous sample
		POSITION pos = m_upHistory_list.FindIndex(samples);
		if(pos == NULL){
			pos = m_upHistory_list.GetTailPosition();
		}

        const TransferredData& latestSample = m_upHistory_list.GetHead();
        const TransferredData& oldestSample = m_upHistory_list.GetAt(pos);
		const uint32 deltaTime = latestSample.timeStamp - oldestSample.timeStamp;
		const uint32 deltaByte = latestSample.dataLength - oldestSample.dataLength;
		nUpDatarate = (deltaTime > 0) ? (1000 * deltaByte / deltaTime) : 0;   // [bytes/s]
    }
    return nUpDatarate;
}

void CUpDownClient::CompUploadRate(){
	// Add new sample
	TransferredData newSample;
	newSample.dataLength = m_nUpDatarateMeasure;
	newSample.timeStamp  = ::GetTickCount();
	m_upHistory_list.AddHead(newSample);

	// Keep up to 11 samples (=> 10 seconds)
	// Keep up to 21 samples (=> 20 seconds)
	while(m_upHistory_list.GetSize() > 21){ // [TPT] Pawcio Bandwith
		m_upHistory_list.RemoveTail();
	}

	if(m_upHistory_list.GetSize() > 1){	// [TPT] - Xman Bugfix
		// Compute datarate (=> display)
		POSITION pos = m_upHistory_list.FindIndex(theApp.glob_prefs->GetDatarateSamples());
		if(pos == NULL){
			pos = m_upHistory_list.GetTailPosition();
		}
		TransferredData& oldestSample = m_upHistory_list.GetAt(pos);
		uint32 deltaTime = newSample.timeStamp - oldestSample.timeStamp;
		uint32 deltaByte = newSample.dataLength - oldestSample.dataLength;
		m_nUpDatarate = (deltaTime > 0) ? (1000 * deltaByte / deltaTime) : 0;   // [bytes/s]
	}

	// Check and then refresh GUI
	m_displayUpDatarateCounter++;
	if(m_displayUpDatarateCounter >= DISPLAY_REFRESH){ // && GetUploadState() == US_UPLOADING){ => already in upload list
		m_displayUpDatarateCounter = 0;
		theApp.emuledlg->transferwnd.uploadlistctrl.RefreshClient(this);
		theApp.emuledlg->transferwnd.clientlistctrl.RefreshClient(this);
	}
}
// Maella end

void CUpDownClient::FlushSendBlocks(){ // call this when you stop upload, or the socket might be not able to send
	bool bBreak = false;
	while (!m_BlockSend_queue.IsEmpty() && m_BlockSend_queue.GetHead()->IsSplitted() && socket && socket->IsConnected() && !bBreak ){	
		Packet* tosend = m_BlockSend_queue.RemoveHead();
		//bool bBreak = tosend->IsLastSplitted();
		theApp.uploadqueue->AddUpDataOverheadOther(tosend->size);
		socket->SendPacket(tosend,true,false);
	}
}

void CUpDownClient::SendHashsetPacket(char* forfileid){
	CKnownFile* file = theApp.sharedfiles->GetFileByID((uchar*)forfileid);
	if (!file)
		throw GetResString(IDS_ERR_REQ_FNF) + _T(" (SendHashsetPacket)");

	CSafeMemFile data(1024);
	data.Write(file->GetFileHash(),16);
	uint16 parts = file->GetHashCount();
	data.Write(&parts,2);
	for (int i = 0; i < parts; i++)
		data.Write(file->GetPartHash(i),16);
	Packet* packet = new Packet(&data);
	packet->opcode = OP_HASHSETANSWER;
	theApp.uploadqueue->AddUpDataOverheadFileRequest(packet->size);
	socket->SendPacket(packet,true,true);
}

void CUpDownClient::ClearUploadBlockRequests(){
	FlushSendBlocks();
	for (POSITION pos = m_BlockRequests_queue.GetHeadPosition();pos != 0;m_BlockRequests_queue.GetNext(pos))
		delete m_BlockRequests_queue.GetAt(pos);
	m_BlockRequests_queue.RemoveAll();
	
	for (POSITION pos = m_DoneBlocks_list.GetHeadPosition();pos != 0;m_DoneBlocks_list.GetNext(pos))
		delete m_DoneBlocks_list.GetAt(pos);
	m_DoneBlocks_list.RemoveAll();
	
	for (POSITION pos = m_BlockSend_queue.GetHeadPosition();pos != 0;m_BlockSend_queue.GetNext(pos))
		delete m_BlockSend_queue.GetAt(pos);
	m_BlockSend_queue.RemoveAll();
}

void CUpDownClient::SendRankingInfo(){
	if (!ExtProtocolAvailable())
		return;
//	uint16 nRank = theApp.uploadqueue->GetWaitingPosition(this);
	uint16 nRank = IsSnafu() ? 1+(rand()%100):theApp.uploadqueue->GetWaitingPosition(this);//eWombat [SNAFU][TPT]
	if (!nRank)
		return;
	Packet* packet = new Packet(OP_QUEUERANKING,12,OP_EMULEPROT);
	MEMSET(packet->pBuffer,0,12);
	MEMCOPY(packet->pBuffer+0,&nRank,2);
	theApp.uploadqueue->AddUpDataOverheadOther(packet->size);
	socket->SendPacket(packet,true,true);
}

void CUpDownClient::SendCommentInfo(CKnownFile *file) {
	// [TPT] - SLUGFILLER: showComments - m_bCommentDirty only applies to the upload file
	if (file == NULL || !ExtProtocolAvailable() || m_byAcceptCommentVer < 1)
		return;
	if (file != theApp.sharedfiles->GetFileByID(requpfileid)){
		if(m_byAcceptCommentVer < 2)	// not supported by remote user
			return;
	} else {
		if (!m_bCommentDirty)	// only send upfile comment once
		return;
	m_bCommentDirty = false;
	}
	// [TPT] - SLUGFILLER: showComments

	int8 rating=file->GetFileRate();
	CString desc=file->GetFileComment();
	if(file->GetFileRate() == 0 && desc.IsEmpty())
		return;

	CSafeMemFile data(256);
	// [TPT] - SLUGFILLER: showComments - send filehash
	if(m_byAcceptCommentVer >= 2)
		data.Write(file->GetFileHash(),16);
	// [TPT] - SLUGFILLER: showComments
	data.Write(&rating,sizeof(rating));
	int length=desc.GetLength();
	if (length>128)
		length=128;
	data.Write(&length,sizeof(length));
	if (length>0)
		data.Write(desc.GetBuffer(),length);
	Packet *packet = new Packet(&data,OP_EMULEPROT);
	packet->opcode = OP_FILEDESC;
	theApp.uploadqueue->AddUpDataOverheadOther(packet->size);
	socket->SendPacket(packet,true);
}

void  CUpDownClient::AddRequestCount(uchar* fileid){
	for (POSITION pos = m_RequestedFiles_list.GetHeadPosition();pos != 0;m_RequestedFiles_list.GetNext(pos)){
		Requested_File_Struct* cur_struct = m_RequestedFiles_list.GetAt(pos);
		if (!md4cmp(cur_struct->fileid,fileid)){
			if (::GetTickCount() - cur_struct->lastasked < MIN_REQUESTTIME && !GetFriendSlot()){ 
				if (GetDownloadState() != DS_DOWNLOADING)
					cur_struct->badrequests++;
				if (cur_struct->badrequests == BADCLIENTBAN){
					Ban();
				}
			}
			else{
				if (cur_struct->badrequests)
					cur_struct->badrequests--;
			}
			cur_struct->lastasked = ::GetTickCount();
			return;
		}
	}
	Requested_File_Struct* new_struct = new Requested_File_Struct;
	MEMSET(new_struct,0,sizeof(Requested_File_Struct));
	md4cpy(new_struct->fileid,fileid);
	new_struct->lastasked = ::GetTickCount();
	m_RequestedFiles_list.AddHead(new_struct);
}


// eWombat [SNAFU][TPT]
/*
void  CUpDownClient::UnBan(){
	theApp.clientlist->AddTrackClient(this);
	theApp.clientlist->RemoveBannedClient( GetIP() );
	SetUploadState(US_NONE);
	ClearWaitStartTime();
	theApp.emuledlg->transferwnd.ShowQueueCount(theApp.uploadqueue->GetWaitingUserCount());
	for (POSITION pos = m_RequestedFiles_list.GetHeadPosition();pos != 0;m_RequestedFiles_list.GetNext(pos)){
		Requested_File_Struct* cur_struct = m_RequestedFiles_list.GetAt(pos);
		cur_struct->badrequests = 0;
		cur_struct->lastasked = 0;	
	}
//	theApp.emuledlg->transferwnd.queuelistctrl.RefreshClient(this, true, true);
}

void CUpDownClient::Ban(){
	theApp.clientlist->AddTrackClient(this);
	if ( !IsBanned() ){
		AddDebugLogLine(false,GetResString(IDS_CLIENTBLOCKED),GetUserName());
	}
#ifdef _DEBUG
	else
		AddDebugLogLine(false,"Ban refreshed for %s (%s) ", GetUserName(), GetFullIP() );
#endif
	theApp.clientlist->AddBannedClient( GetIP() );
	//theApp.uploadqueue->UpdateBanCount();
	SetUploadState(US_BANNED);
	theApp.emuledlg->transferwnd.ShowQueueCount(theApp.uploadqueue->GetWaitingUserCount());
	theApp.emuledlg->transferwnd.queuelistctrl.RefreshClient(this);

}
*/

void CUpDownClient::UDPFileReasked(){
	AddAskedCount();
	SetLastUpRequest();
	uint16 nRank = IsSnafu() ? 1+(rand()%100):theApp.uploadqueue->GetWaitingPosition(this); //eWombat [SNAFU]
	Packet* response = new Packet(OP_REASKACK,2,OP_EMULEPROT);
	MEMCOPY(response->pBuffer,&nRank,2);
	theApp.uploadqueue->AddUpDataOverheadFileRequest(response->size);
	theApp.clientudp->SendPacket(response,GetIP(),GetUDPPort());
}

uint32 CUpDownClient::GetWaitStartTime(){
	if (credits == NULL){
		ASSERT ( false );
		return 0;
	}
	uint32 dwResult = credits->GetSecureWaitStartTime(GetIP());
	if (dwResult > m_dwUploadTime && IsDownloading()){
		//this happens only if two clients with invalid securehash are in the queue - if at all
		dwResult = m_dwUploadTime-1;

		DEBUG_ONLY(AddDebugLogLine(false,"Warning: CUpDownClient::GetWaitStartTime() waittime Collision (%s)",GetUserName()));
	}
	return dwResult;
}

void CUpDownClient::SetWaitStartTime(){
	if (credits == NULL){
		return;
	}
	credits->SetSecWaitStartTime(GetIP());
	m_nSUITimer=::GetTickCount(); // eWombat [SUI][TPT]
}

void CUpDownClient::ClearWaitStartTime(){
	if (credits == NULL){
		return;
	}
	credits->ClearWaitStartTime();
}


bool CUpDownClient::GetFriendSlot(){
	if (credits && theApp.clientcredits->CryptoAvailable()){
		switch(credits->GetCurrentIdentState(GetIP())){
			case IS_IDFAILED:
			case IS_IDNEEDED:
			case IS_IDBADGUY:
				return false;
		}
	}
	return m_bFriendSlot;
}

// [TPT]-Cumulate Bandwidth
void CUpDownClient::ResetProcessRate(){
	cur_ratio = 0.75f;
	avg_ratio = 0.75f;
	for (int i = 0; i < RATIO_AVG; i++)
		ratios[i] = 0.75f;
	ratioIndex = 0;
	m_uBusyTicks=0;
	m_uReadyTicks=0;
}

void CUpDownClient::CalcRatio(bool busy){
	float multiplier = 0.1f;
	if (busy){
		if (m_uBusyTicks < 40)
			multiplier =log10f(m_uBusyTicks) / 2.4f + 0.33f;
		cur_ratio -= cur_ratio/250 * multiplier;
	}
	else{
		if (m_uReadyTicks < 40)
			multiplier = log10f(m_uReadyTicks) / 2.4f + 0.33f;
		cur_ratio += (1.0f-cur_ratio)/250 * multiplier;
	}
	avg_ratio *= RATIO_AVG;
	avg_ratio -= ratios[ratioIndex];
	avg_ratio += cur_ratio;
	avg_ratio /= RATIO_AVG;
	ratios[ratioIndex++] = cur_ratio;
	ratioIndex %= RATIO_AVG;
}

// [TPT]-Cumulate Bandwidth end